# xml.etree test.  This file contains enough tests to make sure that
# all included components work as they should.
# Large parts are extracted from the upstream test suite.

# IMPORTANT: the same doctests are run from "test_xml_etree_c" in
# order to ensure consistency between the C implementation and the
# Python implementation.
#
# For this purpose, the module-level "ET" symbol is temporarily
# monkey-patched when running the "test_xml_etree_c" test suite.
# Don't re-import "xml.etree.ElementTree" module in the docstring,
# except if the test is specific to the Python implementation.

import sys
import cgi

from test import test_support
from test.test_support import findfile

from xml.etree import ElementTree as ET

SIMPLE_XMLFILE = findfile("simple.xml", subdir="xmltestdata")
SIMPLE_NS_XMLFILE = findfile("simple-ns.xml", subdir="xmltestdata")

SAMPLE_XML = """\
<body>
  <tag class='a'>text</tag>
  <tag class='b' />
  <section>
    <tag class='b' id='inner'>subtext</tag>
  </section>
</body>
"""

SAMPLE_SECTION = """\
<section>
  <tag class='b' id='inner'>subtext</tag>
  <nexttag />
  <nextsection>
    <tag />
  </nextsection>
</section>
"""

SAMPLE_XML_NS = """
<body xmlns="http://effbot.org/ns">
  <tag>text</tag>
  <tag />
  <section>
    <tag>subtext</tag>
  </section>
</body>
"""


def sanity():
    """
    Import sanity.

    >>> from xml.etree import ElementTree
    >>> from xml.etree import ElementInclude
    >>> from xml.etree import ElementPath
    """

def check_method(method):
    if not hasattr(method, '__call__'):
        print method, "not callable"

def serialize(elem, to_string=True, **options):
    import StringIO
    file = StringIO.StringIO()
    tree = ET.ElementTree(elem)
    tree.write(file, **options)
    if to_string:
        return file.getvalue()
    else:
        file.seek(0)
        return file

def summarize(elem):
    if elem.tag == ET.Comment:
        return "<Comment>"
    return elem.tag

def summarize_list(seq):
    return [summarize(elem) for elem in seq]

def normalize_crlf(tree):
    for elem in tree.iter():
        if elem.text:
            elem.text = elem.text.replace("\r\n", "\n")
        if elem.tail:
            elem.tail = elem.tail.replace("\r\n", "\n")

def check_string(string):
    len(string)
    for char in string:
        if len(char) != 1:
            print "expected one-character string, got %r" % char
    new_string = string + ""
    new_string = string + " "
    string[:0]

def check_mapping(mapping):
    len(mapping)
    keys = mapping.keys()
    items = mapping.items()
    for key in keys:
        item = mapping[key]
    mapping["key"] = "value"
    if mapping["key"] != "value":
        print "expected value string, got %r" % mapping["key"]

def check_element(element):
    if not ET.iselement(element):
        print "not an element"
    if not hasattr(element, "tag"):
        print "no tag member"
    if not hasattr(element, "attrib"):
        print "no attrib member"
    if not hasattr(element, "text"):
        print "no text member"
    if not hasattr(element, "tail"):
        print "no tail member"

    check_string(element.tag)
    check_mapping(element.attrib)
    if element.text is not None:
        check_string(element.text)
    if element.tail is not None:
        check_string(element.tail)
    for elem in element:
        check_element(elem)

# --------------------------------------------------------------------
# element tree tests

def interface():
    r"""
    Test element tree interface.

    >>> element = ET.Element("tag")
    >>> check_element(element)
    >>> tree = ET.ElementTree(element)
    >>> check_element(tree.getroot())

    >>> element = ET.Element("t\xe4g", key="value")
    >>> tree = ET.ElementTree(element)
    >>> repr(element)   # doctest: +ELLIPSIS
    "<Element 't\\xe4g' at 0x...>"
    >>> element = ET.Element("tag", key="value")

    Make sure all standard element methods exist.

    >>> check_method(element.append)
    >>> check_method(element.extend)
    >>> check_method(element.insert)
    >>> check_method(element.remove)
    >>> check_method(element.getchildren)
    >>> check_method(element.find)
    >>> check_method(element.iterfind)
    >>> check_method(element.findall)
    >>> check_method(element.findtext)
    >>> check_method(element.clear)
    >>> check_method(element.get)
    >>> check_method(element.set)
    >>> check_method(element.keys)
    >>> check_method(element.items)
    >>> check_method(element.iter)
    >>> check_method(element.itertext)
    >>> check_method(element.getiterator)

    These methods return an iterable. See bug 6472.

    >>> check_method(element.iter("tag").next)
    >>> check_method(element.iterfind("tag").next)
    >>> check_method(element.iterfind("*").next)
    >>> check_method(tree.iter("tag").next)
    >>> check_method(tree.iterfind("tag").next)
    >>> check_method(tree.iterfind("*").next)

    These aliases are provided:

    >>> assert ET.XML == ET.fromstring
    >>> assert ET.PI == ET.ProcessingInstruction
    >>> assert ET.XMLParser == ET.XMLTreeBuilder
    """

def simpleops():
    """
    Basic method sanity checks.

    >>> elem = ET.XML("<body><tag/></body>")
    >>> serialize(elem)
    '<body><tag /></body>'
    >>> e = ET.Element("tag2")
    >>> elem.append(e)
    >>> serialize(elem)
    '<body><tag /><tag2 /></body>'
    >>> elem.remove(e)
    >>> serialize(elem)
    '<body><tag /></body>'
    >>> elem.insert(0, e)
    >>> serialize(elem)
    '<body><tag2 /><tag /></body>'
    >>> elem.remove(e)
    >>> elem.extend([e])
    >>> serialize(elem)
    '<body><tag /><tag2 /></body>'
    >>> elem.remove(e)

    >>> element = ET.Element("tag", key="value")
    >>> serialize(element) # 1
    '<tag key="value" />'
    >>> subelement = ET.Element("subtag")
    >>> element.append(subelement)
    >>> serialize(element) # 2
    '<tag key="value"><subtag /></tag>'
    >>> element.insert(0, subelement)
    >>> serialize(element) # 3
    '<tag key="value"><subtag /><subtag /></tag>'
    >>> element.remove(subelement)
    >>> serialize(element) # 4
    '<tag key="value"><subtag /></tag>'
    >>> element.remove(subelement)
    >>> serialize(element) # 5
    '<tag key="value" />'
    >>> element.remove(subelement)
    Traceback (most recent call last):
    ValueError: list.remove(x): x not in list
    >>> serialize(element) # 6
    '<tag key="value" />'
    >>> element[0:0] = [subelement, subelement, subelement]
    >>> serialize(element[1])
    '<subtag />'
    >>> element[1:9] == [element[1], element[2]]
    True
    >>> element[:9:2] == [element[0], element[2]]
    True
    >>> del element[1:2]
    >>> serialize(element)
    '<tag key="value"><subtag /><subtag /></tag>'
    """

def cdata():
    """
    Test CDATA handling (etc).

    >>> serialize(ET.XML("<tag>hello</tag>"))
    '<tag>hello</tag>'
    >>> serialize(ET.XML("<tag>&#104;&#101;&#108;&#108;&#111;</tag>"))
    '<tag>hello</tag>'
    >>> serialize(ET.XML("<tag><![CDATA[hello]]></tag>"))
    '<tag>hello</tag>'
    """

# Only with Python implementation
def simplefind():
    """
    Test find methods using the elementpath fallback.

    >>> from xml.etree import ElementTree

    >>> CurrentElementPath = ElementTree.ElementPath
    >>> ElementTree.ElementPath = ElementTree._SimpleElementPath()
    >>> elem = ElementTree.XML(SAMPLE_XML)
    >>> elem.find("tag").tag
    'tag'
    >>> ElementTree.ElementTree(elem).find("tag").tag
    'tag'
    >>> elem.findtext("tag")
    'text'
    >>> elem.findtext("tog")
    >>> elem.findtext("tog", "default")
    'default'
    >>> ElementTree.ElementTree(elem).findtext("tag")
    'text'
    >>> summarize_list(elem.findall("tag"))
    ['tag', 'tag']
    >>> summarize_list(elem.findall(".//tag"))
    ['tag', 'tag', 'tag']

    Path syntax doesn't work in this case.

    >>> elem.find("section/tag")
    >>> elem.findtext("section/tag")
    >>> summarize_list(elem.findall("section/tag"))
    []

    >>> ElementTree.ElementPath = CurrentElementPath
    """

def find():
    """
    Test find methods (including xpath syntax).

    >>> elem = ET.XML(SAMPLE_XML)
    >>> elem.find("tag").tag
    'tag'
    >>> ET.ElementTree(elem).find("tag").tag
    'tag'
    >>> elem.find("section/tag").tag
    'tag'
    >>> elem.find("./tag").tag
    'tag'
    >>> ET.ElementTree(elem).find("./tag").tag
    'tag'
    >>> ET.ElementTree(elem).find("/tag").tag
    'tag'
    >>> elem[2] = ET.XML(SAMPLE_SECTION)
    >>> elem.find("section/nexttag").tag
    'nexttag'
    >>> ET.ElementTree(elem).find("section/tag").tag
    'tag'
    >>> ET.ElementTree(elem).find("tog")
    >>> ET.ElementTree(elem).find("tog/foo")
    >>> elem.findtext("tag")
    'text'
    >>> elem.findtext("section/nexttag")
    ''
    >>> elem.findtext("section/nexttag", "default")
    ''
    >>> elem.findtext("tog")
    >>> elem.findtext("tog", "default")
    'default'
    >>> ET.ElementTree(elem).findtext("tag")
    'text'
    >>> ET.ElementTree(elem).findtext("tog/foo")
    >>> ET.ElementTree(elem).findtext("tog/foo", "default")
    'default'
    >>> ET.ElementTree(elem).findtext("./tag")
    'text'
    >>> ET.ElementTree(elem).findtext("/tag")
    'text'
    >>> elem.findtext("section/tag")
    'subtext'
    >>> ET.ElementTree(elem).findtext("section/tag")
    'subtext'
    >>> summarize_list(elem.findall("."))
    ['body']
    >>> summarize_list(elem.findall("tag"))
    ['tag', 'tag']
    >>> summarize_list(elem.findall("tog"))
    []
    >>> summarize_list(elem.findall("tog/foo"))
    []
    >>> summarize_list(elem.findall("*"))
    ['tag', 'tag', 'section']
    >>> summarize_list(elem.findall(".//tag"))
    ['tag', 'tag', 'tag', 'tag']
    >>> summarize_list(elem.findall("section/tag"))
    ['tag']
    >>> summarize_list(elem.findall("section//tag"))
    ['tag', 'tag']
    >>> summarize_list(elem.findall("section/*"))
    ['tag', 'nexttag', 'nextsection']
    >>> summarize_list(elem.findall("section//*"))
    ['tag', 'nexttag', 'nextsection', 'tag']
    >>> summarize_list(elem.findall("section/.//*"))
    ['tag', 'nexttag', 'nextsection', 'tag']
    >>> summarize_list(elem.findall("*/*"))
    ['tag', 'nexttag', 'nextsection']
    >>> summarize_list(elem.findall("*//*"))
    ['tag', 'nexttag', 'nextsection', 'tag']
    >>> summarize_list(elem.findall("*/tag"))
    ['tag']
    >>> summarize_list(elem.findall("*/./tag"))
    ['tag']
    >>> summarize_list(elem.findall("./tag"))
    ['tag', 'tag']
    >>> summarize_list(elem.findall(".//tag"))
    ['tag', 'tag', 'tag', 'tag']
    >>> summarize_list(elem.findall("././tag"))
    ['tag', 'tag']
    >>> summarize_list(elem.findall(".//tag[@class]"))
    ['tag', 'tag', 'tag']
    >>> summarize_list(elem.findall(".//tag[@class='a']"))
    ['tag']
    >>> summarize_list(elem.findall(".//tag[@class='b']"))
    ['tag', 'tag']
    >>> summarize_list(elem.findall(".//tag[@id]"))
    ['tag']
    >>> summarize_list(elem.findall(".//section[tag]"))
    ['section']
    >>> summarize_list(elem.findall(".//section[element]"))
    []
    >>> summarize_list(elem.findall("../tag"))
    []
    >>> summarize_list(elem.findall("section/../tag"))
    ['tag', 'tag']
    >>> summarize_list(ET.ElementTree(elem).findall("./tag"))
    ['tag', 'tag']

    Following example is invalid in 1.2.
    A leading '*' is assumed in 1.3.

    >>> elem.findall("section//") == elem.findall("section//*")
    True

    ET's Path module handles this case incorrectly; this gives
    a warning in 1.3, and the behaviour will be modified in 1.4.

    >>> summarize_list(ET.ElementTree(elem).findall("/tag"))
    ['tag', 'tag']

    >>> elem = ET.XML(SAMPLE_XML_NS)
    >>> summarize_list(elem.findall("tag"))
    []
    >>> summarize_list(elem.findall("{http://effbot.org/ns}tag"))
    ['{http://effbot.org/ns}tag', '{http://effbot.org/ns}tag']
    >>> summarize_list(elem.findall(".//{http://effbot.org/ns}tag"))
    ['{http://effbot.org/ns}tag', '{http://effbot.org/ns}tag', '{http://effbot.org/ns}tag']
    """

def file_init():
    """
    >>> import StringIO

    >>> stringfile = StringIO.StringIO(SAMPLE_XML)
    >>> tree = ET.ElementTree(file=stringfile)
    >>> tree.find("tag").tag
    'tag'
    >>> tree.find("section/tag").tag
    'tag'

    >>> tree = ET.ElementTree(file=SIMPLE_XMLFILE)
    >>> tree.find("element").tag
    'element'
    >>> tree.find("element/../empty-element").tag
    'empty-element'
    """

def bad_find():
    """
    Check bad or unsupported path expressions.

    >>> elem = ET.XML(SAMPLE_XML)
    >>> elem.findall("/tag")
    Traceback (most recent call last):
    SyntaxError: cannot use absolute path on element
    """

def path_cache():
    """
    Check that the path cache behaves sanely.

    >>> elem = ET.XML(SAMPLE_XML)
    >>> for i in range(10): ET.ElementTree(elem).find('./'+str(i))
    >>> cache_len_10 = len(ET.ElementPath._cache)
    >>> for i in range(10): ET.ElementTree(elem).find('./'+str(i))
    >>> len(ET.ElementPath._cache) == cache_len_10
    True
    >>> for i in range(20): ET.ElementTree(elem).find('./'+str(i))
    >>> len(ET.ElementPath._cache) > cache_len_10
    True
    >>> for i in range(600): ET.ElementTree(elem).find('./'+str(i))
    >>> len(ET.ElementPath._cache) < 500
    True
    """

def copy():
    """
    Test copy handling (etc).

    >>> import copy
    >>> e1 = ET.XML("<tag>hello<foo/></tag>")
    >>> e2 = copy.copy(e1)
    >>> e3 = copy.deepcopy(e1)
    >>> e1.find("foo").tag = "bar"
    >>> serialize(e1)
    '<tag>hello<bar /></tag>'
    >>> serialize(e2)
    '<tag>hello<bar /></tag>'
    >>> serialize(e3)
    '<tag>hello<foo /></tag>'

    """

def attrib():
    """
    Test attribute handling.

    >>> elem = ET.Element("tag")
    >>> elem.get("key") # 1.1
    >>> elem.get("key", "default") # 1.2
    'default'
    >>> elem.set("key", "value")
    >>> elem.get("key") # 1.3
    'value'

    >>> elem = ET.Element("tag", key="value")
    >>> elem.get("key") # 2.1
    'value'
    >>> elem.attrib # 2.2
    {'key': 'value'}

    >>> attrib = {"key": "value"}
    >>> elem = ET.Element("tag", attrib)
    >>> attrib.clear() # check for aliasing issues
    >>> elem.get("key") # 3.1
    'value'
    >>> elem.attrib # 3.2
    {'key': 'value'}

    >>> attrib = {"key": "value"}
    >>> elem = ET.Element("tag", **attrib)
    >>> attrib.clear() # check for aliasing issues
    >>> elem.get("key") # 4.1
    'value'
    >>> elem.attrib # 4.2
    {'key': 'value'}

    >>> elem = ET.Element("tag", {"key": "other"}, key="value")
    >>> elem.get("key") # 5.1
    'value'
    >>> elem.attrib # 5.2
    {'key': 'value'}

    >>> elem = ET.Element('test')
    >>> elem.text = "aa"
    >>> elem.set('testa', 'testval')
    >>> elem.set('testb', 'test2')
    >>> ET.tostring(elem)
    '<test testa="testval" testb="test2">aa</test>'
    >>> sorted(elem.keys())
    ['testa', 'testb']
    >>> sorted(elem.items())
    [('testa', 'testval'), ('testb', 'test2')]
    >>> elem.attrib['testb']
    'test2'
    >>> elem.attrib['testb'] = 'test1'
    >>> elem.attrib['testc'] = 'test2'
    >>> ET.tostring(elem)
    '<test testa="testval" testb="test1" testc="test2">aa</test>'
    """

def makeelement():
    """
    Test makeelement handling.

    >>> elem = ET.Element("tag")
    >>> attrib = {"key": "value"}
    >>> subelem = elem.makeelement("subtag", attrib)
    >>> if subelem.attrib is attrib:
    ...     print "attrib aliasing"
    >>> elem.append(subelem)
    >>> serialize(elem)
    '<tag><subtag key="value" /></tag>'

    >>> elem.clear()
    >>> serialize(elem)
    '<tag />'
    >>> elem.append(subelem)
    >>> serialize(elem)
    '<tag><subtag key="value" /></tag>'
    >>> elem.extend([subelem, subelem])
    >>> serialize(elem)
    '<tag><subtag key="value" /><subtag key="value" /><subtag key="value" /></tag>'
    >>> elem[:] = [subelem]
    >>> serialize(elem)
    '<tag><subtag key="value" /></tag>'
    >>> elem[:] = tuple([subelem])
    >>> serialize(elem)
    '<tag><subtag key="value" /></tag>'

    """

def parsefile():
    """
    Test parsing from file.

    >>> tree = ET.parse(SIMPLE_XMLFILE)
    >>> normalize_crlf(tree)
    >>> tree.write(sys.stdout)
    <root>
       <element key="value">text</element>
       <element>text</element>tail
       <empty-element />
    </root>
    >>> tree = ET.parse(SIMPLE_NS_XMLFILE)
    >>> normalize_crlf(tree)
    >>> tree.write(sys.stdout)
    <ns0:root xmlns:ns0="namespace">
       <ns0:element key="value">text</ns0:element>
       <ns0:element>text</ns0:element>tail
       <ns0:empty-element />
    </ns0:root>

    >>> with open(SIMPLE_XMLFILE) as f:
    ...     data = f.read()

    >>> parser = ET.XMLParser()
    >>> parser.version  # doctest: +ELLIPSIS
    'Expat ...'
    >>> parser.feed(data)
    >>> print serialize(parser.close())
    <root>
       <element key="value">text</element>
       <element>text</element>tail
       <empty-element />
    </root>

    >>> parser = ET.XMLTreeBuilder() # 1.2 compatibility
    >>> parser.feed(data)
    >>> print serialize(parser.close())
    <root>
       <element key="value">text</element>
       <element>text</element>tail
       <empty-element />
    </root>

    >>> target = ET.TreeBuilder()
    >>> parser = ET.XMLParser(target=target)
    >>> parser.feed(data)
    >>> print serialize(parser.close())
    <root>
       <element key="value">text</element>
       <element>text</element>tail
       <empty-element />
    </root>
    """

def parseliteral():
    """
    >>> element = ET.XML("<html><body>text</body></html>")
    >>> ET.ElementTree(element).write(sys.stdout)
    <html><body>text</body></html>
    >>> element = ET.fromstring("<html><body>text</body></html>")
    >>> ET.ElementTree(element).write(sys.stdout)
    <html><body>text</body></html>
    >>> sequence = ["<html><body>", "text</bo", "dy></html>"]
    >>> element = ET.fromstringlist(sequence)
    >>> print ET.tostring(element)
    <html><body>text</body></html>
    >>> print "".join(ET.tostringlist(element))
    <html><body>text</body></html>
    >>> ET.tostring(element, "ascii")
    "<?xml version='1.0' encoding='ascii'?>\\n<html><body>text</body></html>"
    >>> _, ids = ET.XMLID("<html><body>text</body></html>")
    >>> len(ids)
    0
    >>> _, ids = ET.XMLID("<html><body id='body'>text</body></html>")
    >>> len(ids)
    1
    >>> ids["body"].tag
    'body'
    """

def iterparse():
    """
    Test iterparse interface.

    >>> iterparse = ET.iterparse

    >>> context = iterparse(SIMPLE_XMLFILE)
    >>> action, elem = next(context)
    >>> print action, elem.tag
    end element
    >>> for action, elem in context:
    ...   print action, elem.tag
    end element
    end empty-element
    end root
    >>> context.root.tag
    'root'

    >>> context = iterparse(SIMPLE_NS_XMLFILE)
    >>> for action, elem in context:
    ...   print action, elem.tag
    end {namespace}element
    end {namespace}element
    end {namespace}empty-element
    end {namespace}root

    >>> events = ()
    >>> context = iterparse(SIMPLE_XMLFILE, events)
    >>> for action, elem in context:
    ...   print action, elem.tag

    >>> events = ()
    >>> context = iterparse(SIMPLE_XMLFILE, events=events)
    >>> for action, elem in context:
    ...   print action, elem.tag

    >>> events = ("start", "end")
    >>> context = iterparse(SIMPLE_XMLFILE, events)
    >>> for action, elem in context:
    ...   print action, elem.tag
    start root
    start element
    end element
    start element
    end element
    start empty-element
    end empty-element
    end root

    >>> events = ("start", "end", "start-ns", "end-ns")
    >>> context = iterparse(SIMPLE_NS_XMLFILE, events)
    >>> for action, elem in context:
    ...   if action in ("start", "end"):
    ...     print action, elem.tag
    ...   else:
    ...     print action, elem
    start-ns ('', 'namespace')
    start {namespace}root
    start {namespace}element
    end {namespace}element
    start {namespace}element
    end {namespace}element
    start {namespace}empty-element
    end {namespace}empty-element
    end {namespace}root
    end-ns None

    >>> import StringIO

    >>> events = ('start-ns', 'end-ns')
    >>> context = ET.iterparse(StringIO.StringIO(r"<root xmlns=''/>"), events)
    >>> for action, elem in context:
    ...   print action, elem
    start-ns ('', '')
    end-ns None

    >>> events = ("start", "end", "bogus")
    >>> with open(SIMPLE_XMLFILE, "rb") as f:
    ...     iterparse(f, events)
    Traceback (most recent call last):
    ValueError: unknown event 'bogus'

    >>> source = StringIO.StringIO(
    ...     "<?xml version='1.0' encoding='iso-8859-1'?>\\n"
    ...     "<body xmlns='http://&#233;ffbot.org/ns'\\n"
    ...     "      xmlns:cl\\xe9='http://effbot.org/ns'>text</body>\\n")
    >>> events = ("start-ns",)
    >>> context = iterparse(source, events)
    >>> for action, elem in context:
    ...     print action, elem
    start-ns ('', u'http://\\xe9ffbot.org/ns')
    start-ns (u'cl\\xe9', 'http://effbot.org/ns')

    >>> source = StringIO.StringIO("<document />junk")
    >>> try:
    ...   for action, elem in iterparse(source):
    ...     print action, elem.tag
    ... except ET.ParseError, v:
    ...   print v
    end document
    junk after document element: line 1, column 12
    """

def writefile():
    """
    >>> elem = ET.Element("tag")
    >>> elem.text = "text"
    >>> serialize(elem)
    '<tag>text</tag>'
    >>> ET.SubElement(elem, "subtag").text = "subtext"
    >>> serialize(elem)
    '<tag>text<subtag>subtext</subtag></tag>'

    Test tag suppression
    >>> elem.tag = None
    >>> serialize(elem)
    'text<subtag>subtext</subtag>'
    >>> elem.insert(0, ET.Comment("comment"))
    >>> serialize(elem)     # assumes 1.3
    'text<!--comment--><subtag>subtext</subtag>'
    >>> elem[0] = ET.PI("key", "value")
    >>> serialize(elem)
    'text<?key value?><subtag>subtext</subtag>'
    """

def custom_builder():
    """
    Test parser w. custom builder.

    >>> with open(SIMPLE_XMLFILE) as f:
    ...     data = f.read()
    >>> class Builder:
    ...     def start(self, tag, attrib):
    ...         print "start", tag
    ...     def end(self, tag):
    ...         print "end", tag
    ...     def data(self, text):
    ...         pass
    >>> builder = Builder()
    >>> parser = ET.XMLParser(target=builder)
    >>> parser.feed(data)
    start root
    start element
    end element
    start element
    end element
    start empty-element
    end empty-element
    end root

    >>> with open(SIMPLE_NS_XMLFILE) as f:
    ...     data = f.read()
    >>> class Builder:
    ...     def start(self, tag, attrib):
    ...         print "start", tag
    ...     def end(self, tag):
    ...         print "end", tag
    ...     def data(self, text):
    ...         pass
    ...     def pi(self, target, data):
    ...         print "pi", target, repr(data)
    ...     def comment(self, data):
    ...         print "comment", repr(data)
    >>> builder = Builder()
    >>> parser = ET.XMLParser(target=builder)
    >>> parser.feed(data)
    pi pi 'data'
    comment ' comment '
    start {namespace}root
    start {namespace}element
    end {namespace}element
    start {namespace}element
    end {namespace}element
    start {namespace}empty-element
    end {namespace}empty-element
    end {namespace}root

    """

def getchildren():
    """
    Test Element.getchildren()

    >>> with open(SIMPLE_XMLFILE, "r") as f:
    ...     tree = ET.parse(f)
    >>> for elem in tree.getroot().iter():
    ...     summarize_list(elem.getchildren())
    ['element', 'element', 'empty-element']
    []
    []
    []
    >>> for elem in tree.getiterator():
    ...     summarize_list(elem.getchildren())
    ['element', 'element', 'empty-element']
    []
    []
    []

    >>> elem = ET.XML(SAMPLE_XML)
    >>> len(elem.getchildren())
    3
    >>> len(elem[2].getchildren())
    1
    >>> elem[:] == elem.getchildren()
    True
    >>> child1 = elem[0]
    >>> child2 = elem[2]
    >>> del elem[1:2]
    >>> len(elem.getchildren())
    2
    >>> child1 == elem[0]
    True
    >>> child2 == elem[1]
    True
    >>> elem[0:2] = [child2, child1]
    >>> child2 == elem[0]
    True
    >>> child1 == elem[1]
    True
    >>> child1 == elem[0]
    False
    >>> elem.clear()
    >>> elem.getchildren()
    []
    """

def writestring():
    """
    >>> elem = ET.XML("<html><body>text</body></html>")
    >>> ET.tostring(elem)
    '<html><body>text</body></html>'
    >>> elem = ET.fromstring("<html><body>text</body></html>")
    >>> ET.tostring(elem)
    '<html><body>text</body></html>'
    """

def check_encoding(encoding):
    """
    >>> check_encoding("ascii")
    >>> check_encoding("us-ascii")
    >>> check_encoding("iso-8859-1")
    >>> check_encoding("iso-8859-15")
    >>> check_encoding("cp437")
    >>> check_encoding("mac-roman")
    >>> check_encoding("gbk")
    Traceback (most recent call last):
    ValueError: multi-byte encodings are not supported
    >>> check_encoding("cp037")
    Traceback (most recent call last):
    ParseError: unknown encoding: line 1, column 30
    """
    ET.XML("<?xml version='1.0' encoding='%s'?><xml />" % encoding)

def encoding():
    r"""
    Test encoding issues.

    >>> elem = ET.Element("tag")
    >>> elem.text = u"abc"
    >>> serialize(elem)
    '<tag>abc</tag>'
    >>> serialize(elem, encoding="utf-8")
    '<tag>abc</tag>'
    >>> serialize(elem, encoding="us-ascii")
    '<tag>abc</tag>'
    >>> serialize(elem, encoding="iso-8859-1")
    "<?xml version='1.0' encoding='iso-8859-1'?>\n<tag>abc</tag>"

    >>> elem.text = "<&\"\'>"
    >>> serialize(elem)
    '<tag>&lt;&amp;"\'&gt;</tag>'
    >>> serialize(elem, encoding="utf-8")
    '<tag>&lt;&amp;"\'&gt;</tag>'
    >>> serialize(elem, encoding="us-ascii") # cdata characters
    '<tag>&lt;&amp;"\'&gt;</tag>'
    >>> serialize(elem, encoding="iso-8859-1")
    '<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>\n<tag>&lt;&amp;"\'&gt;</tag>'

    >>> elem.attrib["key"] = "<&\"\'>"
    >>> elem.text = None
    >>> serialize(elem)
    '<tag key="&lt;&amp;&quot;\'&gt;" />'
    >>> serialize(elem, encoding="utf-8")
    '<tag key="&lt;&amp;&quot;\'&gt;" />'
    >>> serialize(elem, encoding="us-ascii")
    '<tag key="&lt;&amp;&quot;\'&gt;" />'
    >>> serialize(elem, encoding="iso-8859-1")
    '<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>\n<tag key="&lt;&amp;&quot;\'&gt;" />'

    >>> elem.text = u'\xe5\xf6\xf6<>'
    >>> elem.attrib.clear()
    >>> serialize(elem)
    '<tag>&#229;&#246;&#246;&lt;&gt;</tag>'
    >>> serialize(elem, encoding="utf-8")
    '<tag>\xc3\xa5\xc3\xb6\xc3\xb6&lt;&gt;</tag>'
    >>> serialize(elem, encoding="us-ascii")
    '<tag>&#229;&#246;&#246;&lt;&gt;</tag>'
    >>> serialize(elem, encoding="iso-8859-1")
    "<?xml version='1.0' encoding='iso-8859-1'?>\n<tag>\xe5\xf6\xf6&lt;&gt;</tag>"

    >>> elem.attrib["key"] = u'\xe5\xf6\xf6<>'
    >>> elem.text = None
    >>> serialize(elem)
    '<tag key="&#229;&#246;&#246;&lt;&gt;" />'
    >>> serialize(elem, encoding="utf-8")
    '<tag key="\xc3\xa5\xc3\xb6\xc3\xb6&lt;&gt;" />'
    >>> serialize(elem, encoding="us-ascii")
    '<tag key="&#229;&#246;&#246;&lt;&gt;" />'
    >>> serialize(elem, encoding="iso-8859-1")
    '<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>\n<tag key="\xe5\xf6\xf6&lt;&gt;" />'
    """

def methods():
    r"""
    Test serialization methods.

    >>> e = ET.XML("<html><link/><script>1 &lt; 2</script></html>")
    >>> e.tail = "\n"
    >>> serialize(e)
    '<html><link /><script>1 &lt; 2</script></html>\n'
    >>> serialize(e, method=None)
    '<html><link /><script>1 &lt; 2</script></html>\n'
    >>> serialize(e, method="xml")
    '<html><link /><script>1 &lt; 2</script></html>\n'
    >>> serialize(e, method="html")
    '<html><link><script>1 < 2</script></html>\n'
    >>> serialize(e, method="text")
    '1 < 2\n'
    """

def iterators():
    """
    Test iterators.

    >>> e = ET.XML("<html><body>this is a <i>paragraph</i>.</body>..</html>")
    >>> summarize_list(e.iter())
    ['html', 'body', 'i']
    >>> summarize_list(e.find("body").iter())
    ['body', 'i']
    >>> summarize(next(e.iter()))
    'html'
    >>> "".join(e.itertext())
    'this is a paragraph...'
    >>> "".join(e.find("body").itertext())
    'this is a paragraph.'
    >>> next(e.itertext())
    'this is a '

    Method iterparse should return an iterator. See bug 6472.

    >>> sourcefile = serialize(e, to_string=False)
    >>> next(ET.iterparse(sourcefile))  # doctest: +ELLIPSIS
    ('end', <Element 'i' at 0x...>)

    >>> tree = ET.ElementTree(None)
    >>> tree.iter()
    Traceback (most recent call last):
    AttributeError: 'NoneType' object has no attribute 'iter'
    """

ENTITY_XML = """\
<!DOCTYPE points [
<!ENTITY % user-entities SYSTEM 'user-entities.xml'>
%user-entities;
]>
<document>&entity;</document>
"""

def entity():
    """
    Test entity handling.

    1) good entities

    >>> e = ET.XML("<document title='&#x8230;'>test</document>")
    >>> serialize(e)
    '<document title="&#33328;">test</document>'

    2) bad entities

    >>> ET.XML("<document>&entity;</document>")
    Traceback (most recent call last):
    ParseError: undefined entity: line 1, column 10

    >>> ET.XML(ENTITY_XML)
    Traceback (most recent call last):
    ParseError: undefined entity &entity;: line 5, column 10

    3) custom entity

    >>> parser = ET.XMLParser()
    >>> parser.entity["entity"] = "text"
    >>> parser.feed(ENTITY_XML)
    >>> root = parser.close()
    >>> serialize(root)
    '<document>text</document>'
    """

def error(xml):
    """

    Test error handling.

    >>> issubclass(ET.ParseError, SyntaxError)
    True
    >>> error("foo").position
    (1, 0)
    >>> error("<tag>&foo;</tag>").position
    (1, 5)
    >>> error("foobar<").position
    (1, 6)

    """
    try:
        ET.XML(xml)
    except ET.ParseError:
        return sys.exc_value

def namespace():
    """
    Test namespace issues.

    1) xml namespace

    >>> elem = ET.XML("<tag xml:lang='en' />")
    >>> serialize(elem) # 1.1
    '<tag xml:lang="en" />'

    2) other "well-known" namespaces

    >>> elem = ET.XML("<rdf:RDF xmlns:rdf='http://www.w3.org/1999/02/22-rdf-syntax-ns#' />")
    >>> serialize(elem) # 2.1
    '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" />'

    >>> elem = ET.XML("<html:html xmlns:html='http://www.w3.org/1999/xhtml' />")
    >>> serialize(elem) # 2.2
    '<html:html xmlns:html="http://www.w3.org/1999/xhtml" />'

    >>> elem = ET.XML("<soap:Envelope xmlns:soap='http://schemas.xmlsoap.org/soap/envelope' />")
    >>> serialize(elem) # 2.3
    '<ns0:Envelope xmlns:ns0="http://schemas.xmlsoap.org/soap/envelope" />'

    3) unknown namespaces
    >>> elem = ET.XML(SAMPLE_XML_NS)
    >>> print serialize(elem)
    <ns0:body xmlns:ns0="http://effbot.org/ns">
      <ns0:tag>text</ns0:tag>
      <ns0:tag />
      <ns0:section>
        <ns0:tag>subtext</ns0:tag>
      </ns0:section>
    </ns0:body>
    """

def qname():
    """
    Test QName handling.

    1) decorated tags

    >>> elem = ET.Element("{uri}tag")
    >>> serialize(elem) # 1.1
    '<ns0:tag xmlns:ns0="uri" />'
    >>> elem = ET.Element(ET.QName("{uri}tag"))
    >>> serialize(elem) # 1.2
    '<ns0:tag xmlns:ns0="uri" />'
    >>> elem = ET.Element(ET.QName("uri", "tag"))
    >>> serialize(elem) # 1.3
    '<ns0:tag xmlns:ns0="uri" />'
    >>> elem = ET.Element(ET.QName("uri", "tag"))
    >>> subelem = ET.SubElement(elem, ET.QName("uri", "tag1"))
    >>> subelem = ET.SubElement(elem, ET.QName("uri", "tag2"))
    >>> serialize(elem) # 1.4
    '<ns0:tag xmlns:ns0="uri"><ns0:tag1 /><ns0:tag2 /></ns0:tag>'

    2) decorated attributes

    >>> elem.clear()
    >>> elem.attrib["{uri}key"] = "value"
    >>> serialize(elem) # 2.1
    '<ns0:tag xmlns:ns0="uri" ns0:key="value" />'

    >>> elem.clear()
    >>> elem.attrib[ET.QName("{uri}key")] = "value"
    >>> serialize(elem) # 2.2
    '<ns0:tag xmlns:ns0="uri" ns0:key="value" />'

    3) decorated values are not converted by default, but the
       QName wrapper can be used for values

    >>> elem.clear()
    >>> elem.attrib["{uri}key"] = "{uri}value"
    >>> serialize(elem) # 3.1
    '<ns0:tag xmlns:ns0="uri" ns0:key="{uri}value" />'

    >>> elem.clear()
    >>> elem.attrib["{uri}key"] = ET.QName("{uri}value")
    >>> serialize(elem) # 3.2
    '<ns0:tag xmlns:ns0="uri" ns0:key="ns0:value" />'

    >>> elem.clear()
    >>> subelem = ET.Element("tag")
    >>> subelem.attrib["{uri1}key"] = ET.QName("{uri2}value")
    >>> elem.append(subelem)
    >>> elem.append(subelem)
    >>> serialize(elem) # 3.3
    '<ns0:tag xmlns:ns0="uri" xmlns:ns1="uri1" xmlns:ns2="uri2"><tag ns1:key="ns2:value" /><tag ns1:key="ns2:value" /></ns0:tag>'

    4) Direct QName tests

    >>> str(ET.QName('ns', 'tag'))
    '{ns}tag'
    >>> str(ET.QName('{ns}tag'))
    '{ns}tag'
    >>> q1 = ET.QName('ns', 'tag')
    >>> q2 = ET.QName('ns', 'tag')
    >>> q1 == q2
    True
    >>> q2 = ET.QName('ns', 'other-tag')
    >>> q1 == q2
    False
    >>> q1 == 'ns:tag'
    False
    >>> q1 == '{ns}tag'
    True
    """

def doctype_public():
    """
    Test PUBLIC doctype.

    >>> elem = ET.XML('<!DOCTYPE html PUBLIC'
    ...   ' "-//W3C//DTD XHTML 1.0 Transitional//EN"'
    ...   ' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
    ...   '<html>text</html>')

    """

def xpath_tokenizer(p):
    """
    Test the XPath tokenizer.

    >>> # tests from the xml specification
    >>> xpath_tokenizer("*")
    ['*']
    >>> xpath_tokenizer("text()")
    ['text', '()']
    >>> xpath_tokenizer("@name")
    ['@', 'name']
    >>> xpath_tokenizer("@*")
    ['@', '*']
    >>> xpath_tokenizer("para[1]")
    ['para', '[', '1', ']']
    >>> xpath_tokenizer("para[last()]")
    ['para', '[', 'last', '()', ']']
    >>> xpath_tokenizer("*/para")
    ['*', '/', 'para']
    >>> xpath_tokenizer("/doc/chapter[5]/section[2]")
    ['/', 'doc', '/', 'chapter', '[', '5', ']', '/', 'section', '[', '2', ']']
    >>> xpath_tokenizer("chapter//para")
    ['chapter', '//', 'para']
    >>> xpath_tokenizer("//para")
    ['//', 'para']
    >>> xpath_tokenizer("//olist/item")
    ['//', 'olist', '/', 'item']
    >>> xpath_tokenizer(".")
    ['.']
    >>> xpath_tokenizer(".//para")
    ['.', '//', 'para']
    >>> xpath_tokenizer("..")
    ['..']
    >>> xpath_tokenizer("../@lang")
    ['..', '/', '@', 'lang']
    >>> xpath_tokenizer("chapter[title]")
    ['chapter', '[', 'title', ']']
    >>> xpath_tokenizer("employee[@secretary and @assistant]")
    ['employee', '[', '@', 'secretary', '', 'and', '', '@', 'assistant', ']']

    >>> # additional tests
    >>> xpath_tokenizer("{http://spam}egg")
    ['{http://spam}egg']
    >>> xpath_tokenizer("./spam.egg")
    ['.', '/', 'spam.egg']
    >>> xpath_tokenizer(".//{http://spam}egg")
    ['.', '//', '{http://spam}egg']
    """
    from xml.etree import ElementPath
    out = []
    for op, tag in ElementPath.xpath_tokenizer(p):
        out.append(op or tag)
    return out

def processinginstruction():
    """
    Test ProcessingInstruction directly

    >>> ET.tostring(ET.ProcessingInstruction('test', 'instruction'))
    '<?test instruction?>'
    >>> ET.tostring(ET.PI('test', 'instruction'))
    '<?test instruction?>'

    Issue #2746

    >>> ET.tostring(ET.PI('test', '<testing&>'))
    '<?test <testing&>?>'
    >>> ET.tostring(ET.PI('test', u'<testing&>\xe3'), 'latin1')
    "<?xml version='1.0' encoding='latin1'?>\\n<?test <testing&>\\xe3?>"
    """

#
# xinclude tests (samples from appendix C of the xinclude specification)

XINCLUDE = {}

XINCLUDE["C1.xml"] = """\
<?xml version='1.0'?>
<document xmlns:xi="http://www.w3.org/2001/XInclude">
  <p>120 Mz is adequate for an average home user.</p>
  <xi:include href="disclaimer.xml"/>
</document>
"""

XINCLUDE["disclaimer.xml"] = """\
<?xml version='1.0'?>
<disclaimer>
  <p>The opinions represented herein represent those of the individual
  and should not be interpreted as official policy endorsed by this
  organization.</p>
</disclaimer>
"""

XINCLUDE["C2.xml"] = """\
<?xml version='1.0'?>
<document xmlns:xi="http://www.w3.org/2001/XInclude">
  <p>This document has been accessed
  <xi:include href="count.txt" parse="text"/> times.</p>
</document>
"""

XINCLUDE["count.txt"] = "324387"

XINCLUDE["C2b.xml"] = """\
<?xml version='1.0'?>
<document xmlns:xi="http://www.w3.org/2001/XInclude">
  <p>This document has been <em>accessed</em>
  <xi:include href="count.txt" parse="text"/> times.</p>
</document>
"""

XINCLUDE["C3.xml"] = """\
<?xml version='1.0'?>
<document xmlns:xi="http://www.w3.org/2001/XInclude">
  <p>The following is the source of the "data.xml" resource:</p>
  <example><xi:include href="data.xml" parse="text"/></example>
</document>
"""

XINCLUDE["data.xml"] = """\
<?xml version='1.0'?>
<data>
  <item><![CDATA[Brooks & Shields]]></item>
</data>
"""

XINCLUDE["C5.xml"] = """\
<?xml version='1.0'?>
<div xmlns:xi="http://www.w3.org/2001/XInclude">
  <xi:include href="example.txt" parse="text">
    <xi:fallback>
      <xi:include href="fallback-example.txt" parse="text">
        <xi:fallback><a href="mailto:bob@example.org">Report error</a></xi:fallback>
      </xi:include>
    </xi:fallback>
  </xi:include>
</div>
"""

XINCLUDE["default.xml"] = """\
<?xml version='1.0'?>
<document xmlns:xi="http://www.w3.org/2001/XInclude">
  <p>Example.</p>
  <xi:include href="{}"/>
</document>
""".format(cgi.escape(SIMPLE_XMLFILE, True))

def xinclude_loader(href, parse="xml", encoding=None):
    try:
        data = XINCLUDE[href]
    except KeyError:
        raise IOError("resource not found")
    if parse == "xml":
        from xml.etree.ElementTree import XML
        return XML(data)
    return data

def xinclude():
    r"""
    Basic inclusion example (XInclude C.1)

    >>> from xml.etree import ElementTree as ET
    >>> from xml.etree import ElementInclude

    >>> document = xinclude_loader("C1.xml")
    >>> ElementInclude.include(document, xinclude_loader)
    >>> print serialize(document) # C1
    <document>
      <p>120 Mz is adequate for an average home user.</p>
      <disclaimer>
      <p>The opinions represented herein represent those of the individual
      and should not be interpreted as official policy endorsed by this
      organization.</p>
    </disclaimer>
    </document>

    Textual inclusion example (XInclude C.2)

    >>> document = xinclude_loader("C2.xml")
    >>> ElementInclude.include(document, xinclude_loader)
    >>> print serialize(document) # C2
    <document>
      <p>This document has been accessed
      324387 times.</p>
    </document>

    Textual inclusion after sibling element (based on modified XInclude C.2)

    >>> document = xinclude_loader("C2b.xml")
    >>> ElementInclude.include(document, xinclude_loader)
    >>> print(serialize(document)) # C2b
    <document>
      <p>This document has been <em>accessed</em>
      324387 times.</p>
    </document>

    Textual inclusion of XML example (XInclude C.3)

    >>> document = xinclude_loader("C3.xml")
    >>> ElementInclude.include(document, xinclude_loader)
    >>> print serialize(document) # C3
    <document>
      <p>The following is the source of the "data.xml" resource:</p>
      <example>&lt;?xml version='1.0'?&gt;
    &lt;data&gt;
      &lt;item&gt;&lt;![CDATA[Brooks &amp; Shields]]&gt;&lt;/item&gt;
    &lt;/data&gt;
    </example>
    </document>

    Fallback example (XInclude C.5)
    Note! Fallback support is not yet implemented

    >>> document = xinclude_loader("C5.xml")
    >>> ElementInclude.include(document, xinclude_loader)
    Traceback (most recent call last):
    IOError: resource not found
    >>> # print serialize(document) # C5
    """

def xinclude_default():
    """
    >>> from xml.etree import ElementInclude

    >>> document = xinclude_loader("default.xml")
    >>> ElementInclude.include(document)
    >>> print serialize(document) # default
    <document>
      <p>Example.</p>
      <root>
       <element key="value">text</element>
       <element>text</element>tail
       <empty-element />
    </root>
    </document>
    """

#
# badly formatted xi:include tags

XINCLUDE_BAD = {}

XINCLUDE_BAD["B1.xml"] = """\
<?xml version='1.0'?>
<document xmlns:xi="http://www.w3.org/2001/XInclude">
  <p>120 Mz is adequate for an average home user.</p>
  <xi:include href="disclaimer.xml" parse="BAD_TYPE"/>
</document>
"""

XINCLUDE_BAD["B2.xml"] = """\
<?xml version='1.0'?>
<div xmlns:xi="http://www.w3.org/2001/XInclude">
    <xi:fallback></xi:fallback>
</div>
"""

def xinclude_failures():
    r"""
    Test failure to locate included XML file.

    >>> from xml.etree import ElementInclude

    >>> def none_loader(href, parser, encoding=None):
    ...     return None

    >>> document = ET.XML(XINCLUDE["C1.xml"])
    >>> ElementInclude.include(document, loader=none_loader)
    Traceback (most recent call last):
    FatalIncludeError: cannot load 'disclaimer.xml' as 'xml'

    Test failure to locate included text file.

    >>> document = ET.XML(XINCLUDE["C2.xml"])
    >>> ElementInclude.include(document, loader=none_loader)
    Traceback (most recent call last):
    FatalIncludeError: cannot load 'count.txt' as 'text'

    Test bad parse type.

    >>> document = ET.XML(XINCLUDE_BAD["B1.xml"])
    >>> ElementInclude.include(document, loader=none_loader)
    Traceback (most recent call last):
    FatalIncludeError: unknown parse type in xi:include tag ('BAD_TYPE')

    Test xi:fallback outside xi:include.

    >>> document = ET.XML(XINCLUDE_BAD["B2.xml"])
    >>> ElementInclude.include(document, loader=none_loader)
    Traceback (most recent call last):
    FatalIncludeError: xi:fallback tag must be child of xi:include ('{http://www.w3.org/2001/XInclude}fallback')
    """

# --------------------------------------------------------------------
# reported bugs

def bug_xmltoolkit21():
    """

    marshaller gives obscure errors for non-string values

    >>> elem = ET.Element(123)
    >>> serialize(elem) # tag
    Traceback (most recent call last):
    TypeError: cannot serialize 123 (type int)
    >>> elem = ET.Element("elem")
    >>> elem.text = 123
    >>> serialize(elem) # text
    Traceback (most recent call last):
    TypeError: cannot serialize 123 (type int)
    >>> elem = ET.Element("elem")
    >>> elem.tail = 123
    >>> serialize(elem) # tail
    Traceback (most recent call last):
    TypeError: cannot serialize 123 (type int)
    >>> elem = ET.Element("elem")
    >>> elem.set(123, "123")
    >>> serialize(elem) # attribute key
    Traceback (most recent call last):
    TypeError: cannot serialize 123 (type int)
    >>> elem = ET.Element("elem")
    >>> elem.set("123", 123)
    >>> serialize(elem) # attribute value
    Traceback (most recent call last):
    TypeError: cannot serialize 123 (type int)

    """

def bug_xmltoolkit25():
    """

    typo in ElementTree.findtext

    >>> elem = ET.XML(SAMPLE_XML)
    >>> tree = ET.ElementTree(elem)
    >>> tree.findtext("tag")
    'text'
    >>> tree.findtext("section/tag")
    'subtext'

    """

def bug_xmltoolkit28():
    """

    .//tag causes exceptions

    >>> tree = ET.XML("<doc><table><tbody/></table></doc>")
    >>> summarize_list(tree.findall(".//thead"))
    []
    >>> summarize_list(tree.findall(".//tbody"))
    ['tbody']

    """

def bug_xmltoolkitX1():
    """

    dump() doesn't flush the output buffer

    >>> tree = ET.XML("<doc><table><tbody/></table></doc>")
    >>> ET.dump(tree); sys.stdout.write("tail")
    <doc><table><tbody /></table></doc>
    tail

    """

def bug_xmltoolkit39():
    """

    non-ascii element and attribute names doesn't work

    >>> tree = ET.XML("<?xml version='1.0' encoding='iso-8859-1'?><t\xe4g />")
    >>> ET.tostring(tree, "utf-8")
    '<t\\xc3\\xa4g />'

    >>> tree = ET.XML("<?xml version='1.0' encoding='iso-8859-1'?><tag \xe4ttr='v&#228;lue' />")
    >>> tree.attrib
    {u'\\xe4ttr': u'v\\xe4lue'}
    >>> ET.tostring(tree, "utf-8")
    '<tag \\xc3\\xa4ttr="v\\xc3\\xa4lue" />'

    >>> tree = ET.XML("<?xml version='1.0' encoding='iso-8859-1'?><t\xe4g>text</t\xe4g>")
    >>> ET.tostring(tree, "utf-8")
    '<t\\xc3\\xa4g>text</t\\xc3\\xa4g>'

    >>> tree = ET.Element(u"t\u00e4g")
    >>> ET.tostring(tree, "utf-8")
    '<t\\xc3\\xa4g />'

    >>> tree = ET.Element("tag")
    >>> tree.set(u"\u00e4ttr", u"v\u00e4lue")
    >>> ET.tostring(tree, "utf-8")
    '<tag \\xc3\\xa4ttr="v\\xc3\\xa4lue" />'

    """

def bug_xmltoolkit54():
    """

    problems handling internally defined entities

    >>> e = ET.XML("<!DOCTYPE doc [<!ENTITY ldots '&#x8230;'>]><doc>&ldots;</doc>")
    >>> serialize(e)
    '<doc>&#33328;</doc>'

    """

def bug_xmltoolkit55():
    """

    make sure we're reporting the first error, not the last

    >>> e = ET.XML("<!DOCTYPE doc SYSTEM 'doc.dtd'><doc>&ldots;&ndots;&rdots;</doc>")
    Traceback (most recent call last):
    ParseError: undefined entity &ldots;: line 1, column 36

    """

class ExceptionFile:
    def read(self, x):
        raise IOError

def xmltoolkit60():
    """

    Handle crash in stream source.
    >>> tree = ET.parse(ExceptionFile())
    Traceback (most recent call last):
    IOError

    """

XMLTOOLKIT62_DOC = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE patent-application-publication SYSTEM "pap-v15-2001-01-31.dtd" []>
<patent-application-publication>
<subdoc-abstract>
<paragraph id="A-0001" lvl="0">A new cultivar of Begonia plant named &lsquo;BCT9801BEG&rsquo;.</paragraph>
</subdoc-abstract>
</patent-application-publication>"""


def xmltoolkit62():
    """

    Don't crash when using custom entities.

    >>> xmltoolkit62()
    u'A new cultivar of Begonia plant named \u2018BCT9801BEG\u2019.'

    """
    ENTITIES = {u'rsquo': u'\u2019', u'lsquo': u'\u2018'}
    parser = ET.XMLTreeBuilder()
    parser.entity.update(ENTITIES)
    parser.feed(XMLTOOLKIT62_DOC)
    t = parser.close()
    return t.find('.//paragraph').text

def xmltoolkit63():
    """

    Check reference leak.
    >>> xmltoolkit63()
    >>> count = sys.getrefcount(None)
    >>> for i in range(1000):
    ...     xmltoolkit63()
    >>> sys.getrefcount(None) - count
    0

    """
    tree = ET.TreeBuilder()
    tree.start("tag", {})
    tree.data("text")
    tree.end("tag")

# --------------------------------------------------------------------


def bug_200708_newline():
    r"""

    Preserve newlines in attributes.

    >>> e = ET.Element('SomeTag', text="def _f():\n  return 3\n")
    >>> ET.tostring(e)
    '<SomeTag text="def _f():&#10;  return 3&#10;" />'
    >>> ET.XML(ET.tostring(e)).get("text")
    'def _f():\n  return 3\n'
    >>> ET.tostring(ET.XML(ET.tostring(e)))
    '<SomeTag text="def _f():&#10;  return 3&#10;" />'

    """

def bug_200708_close():
    """

    Test default builder.
    >>> parser = ET.XMLParser() # default
    >>> parser.feed("<element>some text</element>")
    >>> summarize(parser.close())
    'element'

    Test custom builder.
    >>> class EchoTarget:
    ...     def close(self):
    ...         return ET.Element("element") # simulate root
    >>> parser = ET.XMLParser(EchoTarget())
    >>> parser.feed("<element>some text</element>")
    >>> summarize(parser.close())
    'element'

    """

def bug_200709_default_namespace():
    """

    >>> e = ET.Element("{default}elem")
    >>> s = ET.SubElement(e, "{default}elem")
    >>> serialize(e, default_namespace="default") # 1
    '<elem xmlns="default"><elem /></elem>'

    >>> e = ET.Element("{default}elem")
    >>> s = ET.SubElement(e, "{default}elem")
    >>> s = ET.SubElement(e, "{not-default}elem")
    >>> serialize(e, default_namespace="default") # 2
    '<elem xmlns="default" xmlns:ns1="not-default"><elem /><ns1:elem /></elem>'

    >>> e = ET.Element("{default}elem")
    >>> s = ET.SubElement(e, "{default}elem")
    >>> s = ET.SubElement(e, "elem") # unprefixed name
    >>> serialize(e, default_namespace="default") # 3
    Traceback (most recent call last):
    ValueError: cannot use non-qualified names with default_namespace option

    """

def bug_200709_register_namespace():
    """

    >>> ET.tostring(ET.Element("{http://namespace.invalid/does/not/exist/}title"))
    '<ns0:title xmlns:ns0="http://namespace.invalid/does/not/exist/" />'
    >>> ET.register_namespace("foo", "http://namespace.invalid/does/not/exist/")
    >>> ET.tostring(ET.Element("{http://namespace.invalid/does/not/exist/}title"))
    '<foo:title xmlns:foo="http://namespace.invalid/does/not/exist/" />'

    And the Dublin Core namespace is in the default list:

    >>> ET.tostring(ET.Element("{http://purl.org/dc/elements/1.1/}title"))
    '<dc:title xmlns:dc="http://purl.org/dc/elements/1.1/" />'

    """

def bug_200709_element_comment():
    """

    Not sure if this can be fixed, really (since the serializer needs
    ET.Comment, not cET.comment).

    >>> a = ET.Element('a')
    >>> a.append(ET.Comment('foo'))
    >>> a[0].tag == ET.Comment
    True

    >>> a = ET.Element('a')
    >>> a.append(ET.PI('foo'))
    >>> a[0].tag == ET.PI
    True

    """

def bug_200709_element_insert():
    """

    >>> a = ET.Element('a')
    >>> b = ET.SubElement(a, 'b')
    >>> c = ET.SubElement(a, 'c')
    >>> d = ET.Element('d')
    >>> a.insert(0, d)
    >>> summarize_list(a)
    ['d', 'b', 'c']
    >>> a.insert(-1, d)
    >>> summarize_list(a)
    ['d', 'b', 'd', 'c']

    """

def bug_200709_iter_comment():
    """

    >>> a = ET.Element('a')
    >>> b = ET.SubElement(a, 'b')
    >>> comment_b = ET.Comment("TEST-b")
    >>> b.append(comment_b)
    >>> summarize_list(a.iter(ET.Comment))
    ['<Comment>']

    """

def bug_18347():
    """

    >>> e = ET.XML('<html><CamelCase>text</CamelCase></html>')
    >>> serialize(e)
    '<html><CamelCase>text</CamelCase></html>'
    >>> serialize(e, method="html")
    '<html><CamelCase>text</CamelCase></html>'
    """

# --------------------------------------------------------------------
# reported on bugs.python.org

def bug_1534630():
    """

    >>> bob = ET.TreeBuilder()
    >>> e = bob.data("data")
    >>> e = bob.start("tag", {})
    >>> e = bob.end("tag")
    >>> e = bob.close()
    >>> serialize(e)
    '<tag />'

    """

def check_issue6233():
    """

    >>> e = ET.XML("<?xml version='1.0' encoding='utf-8'?><body>t\\xc3\\xa3g</body>")
    >>> ET.tostring(e, 'ascii')
    "<?xml version='1.0' encoding='ascii'?>\\n<body>t&#227;g</body>"
    >>> e = ET.XML("<?xml version='1.0' encoding='iso-8859-1'?><body>t\\xe3g</body>")
    >>> ET.tostring(e, 'ascii')
    "<?xml version='1.0' encoding='ascii'?>\\n<body>t&#227;g</body>"

    """

def check_issue3151():
    """

    >>> e = ET.XML('<prefix:localname xmlns:prefix="${stuff}"/>')
    >>> e.tag
    '{${stuff}}localname'
    >>> t = ET.ElementTree(e)
    >>> ET.tostring(e)
    '<ns0:localname xmlns:ns0="${stuff}" />'

    """

def check_issue6565():
    """

    >>> elem = ET.XML("<body><tag/></body>")
    >>> summarize_list(elem)
    ['tag']
    >>> newelem = ET.XML(SAMPLE_XML)
    >>> elem[:] = newelem[:]
    >>> summarize_list(elem)
    ['tag', 'tag', 'section']

    """

def check_html_empty_elems_serialization(self):
    # issue 15970
    # from http://www.w3.org/TR/html401/index/elements.html
    """

    >>> empty_elems = ['AREA', 'BASE', 'BASEFONT', 'BR', 'COL', 'FRAME', 'HR',
    ...                'IMG', 'INPUT', 'ISINDEX', 'LINK', 'META', 'PARAM']
    >>> elems = ''.join('<%s />' % elem for elem in empty_elems)
    >>> serialize(ET.XML('<html>%s</html>' % elems), method='html')
    '<html><AREA><BASE><BASEFONT><BR><COL><FRAME><HR><IMG><INPUT><ISINDEX><LINK><META><PARAM></html>'
    >>> serialize(ET.XML('<html>%s</html>' % elems.lower()), method='html')
    '<html><area><base><basefont><br><col><frame><hr><img><input><isindex><link><meta><param></html>'
    >>> elems = ''.join('<%s></%s>' % (elem, elem) for elem in empty_elems)
    >>> serialize(ET.XML('<html>%s</html>' % elems), method='html')
    '<html><AREA><BASE><BASEFONT><BR><COL><FRAME><HR><IMG><INPUT><ISINDEX><LINK><META><PARAM></html>'
    >>> serialize(ET.XML('<html>%s</html>' % elems.lower()), method='html')
    '<html><area><base><basefont><br><col><frame><hr><img><input><isindex><link><meta><param></html>'

    """

# --------------------------------------------------------------------


class CleanContext(object):
    """Provide default namespace mapping and path cache."""
    checkwarnings = None

    def __init__(self, quiet=False):
        if sys.flags.optimize >= 2:
            # under -OO, doctests cannot be run and therefore not all warnings
            # will be emitted
            quiet = True
        deprecations = (
            # Search behaviour is broken if search path starts with "/".
            ("This search is broken in 1.3 and earlier, and will be fixed "
             "in a future version.  If you rely on the current behaviour, "
             "change it to '.+'", FutureWarning),
            # Element.getchildren() and Element.getiterator() are deprecated.
            ("This method will be removed in future versions.  "
             "Use .+ instead.", DeprecationWarning),
            ("This method will be removed in future versions.  "
             "Use .+ instead.", PendingDeprecationWarning),
            # XMLParser.doctype() is deprecated.
            ("This method of XMLParser is deprecated.  Define doctype.. "
             "method on the TreeBuilder target.", DeprecationWarning))
        self.checkwarnings = test_support.check_warnings(*deprecations,
                                                         quiet=quiet)

    def __enter__(self):
        from xml.etree import ElementTree
        self._nsmap = ElementTree._namespace_map
        self._path_cache = ElementTree.ElementPath._cache
        # Copy the default namespace mapping
        ElementTree._namespace_map = self._nsmap.copy()
        # Copy the path cache (should be empty)
        ElementTree.ElementPath._cache = self._path_cache.copy()
        self.checkwarnings.__enter__()

    def __exit__(self, *args):
        from xml.etree import ElementTree
        # Restore mapping and path cache
        ElementTree._namespace_map = self._nsmap
        ElementTree.ElementPath._cache = self._path_cache
        self.checkwarnings.__exit__(*args)


def test_main(module_name='xml.etree.ElementTree'):
    from test import test_xml_etree

    use_py_module = (module_name == 'xml.etree.ElementTree')

    # The same doctests are used for both the Python and the C implementations
    assert test_xml_etree.ET.__name__ == module_name

    # XXX the C module should give the same warnings as the Python module
    with CleanContext(quiet=not use_py_module):
        test_support.run_doctest(test_xml_etree, verbosity=True)

    # The module should not be changed by the tests
    assert test_xml_etree.ET.__name__ == module_name

if __name__ == '__main__':
    test_main()



import os
import os.path

import logging
import toml
import simpleplugin

Inputs = {}
LoadedInput = {}
loadedPluginPrefix = ""

class Config(object):
    def __init__(self):
        self.Inputs = {}

    def load(self, parsed):
        self.Inputs = parsed.get('inputs')

def getAllInputs():
    return LoadedInput.keys()

def add(name, creator):
    Inputs[name] = creator

def loadPlugins(pluginPrefix):
    global loadedPluginPrefix
    loadedPluginPrefix = pluginPrefix
    all_inputs_this_time = []
    listPluginConfigs(pluginPrefix, lambda configfile: loadConfig(configfile, all_inputs_this_time))
    return all_inputs_this_time

def loadConfig(filePath, all_inputs):
    try:
        with open(filePath, 'r') as f:
            content = f.read()
    except IOError as e:
        logging.debug("Error reading file {}: {}".format(filePath, e))
        return

    conf, parsedDict = validate(content)
    if conf is None:
        logging.debug("Error decoding TOML: {}".format(filePath))
        return
    
    inputs = loadConfigIntoInputs(filePath, parsedDict, conf.Inputs)

    all_inputs.extend(inputs)
    return inputs

def setLoadedInputCache(configPath, name, inst):
    _, only_fileName = os.path.split(configPath)
    LoadedInput[only_fileName] = simpleplugin.LoadedInput(configPath, inst, name)

def listPluginConfigs(pluginPrefix, h1):
    try:
        for root, _, files in os.walk(pluginPrefix):
            for fileName in files:
                configPath = os.path.join(root, fileName)
                if os.path.isfile(configPath) and os.path.getsize(configPath) > 0:
                    h1(configPath)
    except IOError, e:
        raise Exception("Error walking plugin directory {}: {}".format(pluginPrefix, e))

def loadConfigIntoInputs(filePath, parsedDict, inputConfigs):
    rendered_inputs = []
    
    for name, primitives in inputConfigs.iteritems():
        inputCreator = Inputs.get(name)
        if inputCreator is None:
            logging.error("unknown input {}".format(name))
            continue

        for primitive in primitives:
            inp = inputCreator()

            inp.load(primitive)
            
            rendered_inputs.append(inp)

            setLoadedInputCache(filePath, name, inp)

    return rendered_inputs

def getInputToml(filename):
    loadedInput = LoadedInput.get(filename)
    if loadedInput is not None:
        try:
            with open(loadedInput.ConfigPath, 'r') as f:
                inputToml = f.read()
            return inputToml
        except IOError, e:
            return None

    return None

def setInputToml(inputFile, inputToml):
    conf, parsedDict = validate(inputToml)
    if not conf:
        logging.error("set_inputToml invalid toml for input {}".format(inputFile))
        return
    if inputFile in LoadedInput:
        restartInputService(conf, parsedDict, inputFile)
    else:
        loadConfigIntoInputs(inputFile, parsedDict, conf.Inputs)

    saveConfigFile(inputFile, inputToml)

    return

def deleteInput(inputFile):
    err = removeInputService(inputFile)
    deleteConfigFile(inputFile)
    return err

def hasInput(inputFile):
    return inputFile in LoadedInput

def saveConfigFile(inputFile, inputToml):
    try:
        os.stat(loadedPluginPrefix)
        
        full_path = os.path.join(loadedPluginPrefix, inputFile)

        with open(full_path, 'w') as f:
            f.write(inputToml)
        
        return True
    except IOError as e:
        logging.error("Error writing file {}: {}".format(full_path, e))
        return False
    
def deleteConfigFile(inputFile):
    try:
        os.stat(loadedPluginPrefix)
        full_path = os.path.join(loadedPluginPrefix, inputFile)
        os.remove(full_path)
        return True
    except IOError, e:
        logging.error("Error removing file {}: {}".format(full_path, e))
        return False
    except OSError, e:
        logging.error("Error finding file {}: {}".format(full_path, e))
        return False

def restartInputService(conf, parseddict, inputFile):
    loadedInput = LoadedInput.get(inputFile)
    if loadedInput is not None:
        if loadedInput.Instance is not None and loadedInput.name() in conf.Inputs:
            if isinstance(loadedInput.Instance, simpleplugin.ServiceInput):
                loadedInput.Instance.stop()
                loadedInput.Instance.load(conf.Inputs[loadedInput.name()] )
                loadedInput.Instance.restart()

    else:
        logging.error("input {} not found".format(inputFile))
        return False

    return True

def stopInputService(inputFile):
    loadedInput = LoadedInput.get(inputFile)
    if loadedInput is not None:
        if loadedInput.Instance is not None:
            if isinstance(loadedInput.Instance, simpleplugin.ServiceInput):
                loadedInput.Instance.stop()
    else:
        logging.error("input {} not found".format(inputFile))
        return False

    return True

def removeInputService(inputFile):
    loadedInput = LoadedInput.get(inputFile)
    if loadedInput is not None:
        if loadedInput.Instance is not None:
            if isinstance(loadedInput.Instance, simpleplugin.ServiceInput):
                loadedInput.Instance.stop()
        del LoadedInput[inputFile]
    else:
        logging.error("input {} not found".format(inputFile))
        return False

    return True

def validate(inputToml):
    conf = Config()
    try:
        parsed = toml.loads(inputToml)
        if not parsed.has_key("inputs") and not parsed.has_key(u"inputs") :
            return None, parsed
        conf.load(parsed)
    except Exception, e:
        logging.error("input validation error {}".format(e))
        
        return None, None

    return conf, parsed





import test.test_support, unittest
import os

class CodingTest(unittest.TestCase):
    def test_bad_coding(self):
        module_name = 'bad_coding'
        self.verify_bad_module(module_name)

    def test_bad_coding2(self):
        module_name = 'bad_coding2'
        self.verify_bad_module(module_name)

    def verify_bad_module(self, module_name):
        self.assertRaises(SyntaxError, __import__, 'test.' + module_name)

        path = os.path.dirname(__file__)
        filename = os.path.join(path, module_name + '.py')
        with open(filename) as fp:
            text = fp.read()
        self.assertRaises(SyntaxError, compile, text, filename, 'exec')

    def test_error_from_string(self):
        # See http://bugs.python.org/issue6289
        input = u"# coding: ascii\n\N{SNOWMAN}".encode('utf-8')
        with self.assertRaises(SyntaxError) as c:
            compile(input, "<string>", "exec")
        expected = "'ascii' codec can't decode byte 0xe2 in position 16: " \
                   "ordinal not in range(128)"
        self.assertTrue(c.exception.args[0].startswith(expected))


def test_main():
    test.test_support.run_unittest(CodingTest)

if __name__ == "__main__":
    test_main()



"""High-perfomance logging profiler, mostly written in C."""

import _hotshot
from _hotshot import ProfilerError

from warnings import warnpy3k as _warnpy3k
_warnpy3k("The 'hotshot' module is not supported in 3.x, "
          "use the 'profile' module instead.", stacklevel=2)

class Profile:
    def __init__(self, logfn, lineevents=0, linetimings=1):
        self.lineevents = lineevents and 1 or 0
        self.linetimings = (linetimings and lineevents) and 1 or 0
        self._prof = p = _hotshot.profiler(
            logfn, self.lineevents, self.linetimings)

        # Attempt to avoid confusing results caused by the presence of
        # Python wrappers around these functions, but only if we can
        # be sure the methods have not been overridden or extended.
        if self.__class__ is Profile:
            self.close = p.close
            self.start = p.start
            self.stop = p.stop
            self.addinfo = p.addinfo

    def close(self):
        """Close the logfile and terminate the profiler."""
        self._prof.close()

    def fileno(self):
        """Return the file descriptor of the profiler's log file."""
        return self._prof.fileno()

    def start(self):
        """Start the profiler."""
        self._prof.start()

    def stop(self):
        """Stop the profiler."""
        self._prof.stop()

    def addinfo(self, key, value):
        """Add an arbitrary labelled value to the profile log."""
        self._prof.addinfo(key, value)

    # These methods offer the same interface as the profile.Profile class,
    # but delegate most of the work to the C implementation underneath.

    def run(self, cmd):
        """Profile an exec-compatible string in the script
        environment.

        The globals from the __main__ module are used as both the
        globals and locals for the script.
        """
        import __main__
        dict = __main__.__dict__
        return self.runctx(cmd, dict, dict)

    def runctx(self, cmd, globals, locals):
        """Evaluate an exec-compatible string in a specific
        environment.

        The string is compiled before profiling begins.
        """
        code = compile(cmd, "<string>", "exec")
        self._prof.runcode(code, globals, locals)
        return self

    def runcall(self, func, *args, **kw):
        """Profile a single call of a callable.

        Additional positional and keyword arguments may be passed
        along; the result of the call is returned, and exceptions are
        allowed to propogate cleanly, while ensuring that profiling is
        disabled on the way out.
        """
        return self._prof.runcall(func, args, kw)



"""Unit tests for the memoryview

XXX We need more tests! Some tests are in test_bytes
"""

import unittest
import sys
import gc
import weakref
import array
from test import test_support
import io


class AbstractMemoryTests:
    source_bytes = b"abcdef"

    @property
    def _source(self):
        return self.source_bytes

    @property
    def _types(self):
        return filter(None, [self.ro_type, self.rw_type])

    def check_getitem_with_type(self, tp):
        item = self.getitem_type
        b = tp(self._source)
        oldrefcount = sys.getrefcount(b)
        m = self._view(b)
        self.assertEqual(m[0], item(b"a"))
        self.assertIsInstance(m[0], bytes)
        self.assertEqual(m[5], item(b"f"))
        self.assertEqual(m[-1], item(b"f"))
        self.assertEqual(m[-6], item(b"a"))
        # Bounds checking
        self.assertRaises(IndexError, lambda: m[6])
        self.assertRaises(IndexError, lambda: m[-7])
        self.assertRaises(IndexError, lambda: m[sys.maxsize])
        self.assertRaises(IndexError, lambda: m[-sys.maxsize])
        # Type checking
        self.assertRaises(TypeError, lambda: m[None])
        self.assertRaises(TypeError, lambda: m[0.0])
        self.assertRaises(TypeError, lambda: m["a"])
        m = None
        self.assertEqual(sys.getrefcount(b), oldrefcount)

    def test_getitem(self):
        for tp in self._types:
            self.check_getitem_with_type(tp)

    def test_iter(self):
        for tp in self._types:
            b = tp(self._source)
            m = self._view(b)
            self.assertEqual(list(m), [m[i] for i in range(len(m))])

    def test_repr(self):
        for tp in self._types:
            b = tp(self._source)
            m = self._view(b)
            self.assertIsInstance(m.__repr__(), str)

    def test_setitem_readonly(self):
        if not self.ro_type:
            self.skipTest("no read-only type to test")
        b = self.ro_type(self._source)
        oldrefcount = sys.getrefcount(b)
        m = self._view(b)
        def setitem(value):
            m[0] = value
        self.assertRaises(TypeError, setitem, b"a")
        self.assertRaises(TypeError, setitem, 65)
        self.assertRaises(TypeError, setitem, memoryview(b"a"))
        m = None
        self.assertEqual(sys.getrefcount(b), oldrefcount)

    def test_setitem_writable(self):
        if not self.rw_type:
            self.skipTest("no writable type to test")
        tp = self.rw_type
        b = self.rw_type(self._source)
        oldrefcount = sys.getrefcount(b)
        m = self._view(b)
        m[0] = tp(b"0")
        self._check_contents(tp, b, b"0bcdef")
        m[1:3] = tp(b"12")
        self._check_contents(tp, b, b"012def")
        m[1:1] = tp(b"")
        self._check_contents(tp, b, b"012def")
        m[:] = tp(b"abcdef")
        self._check_contents(tp, b, b"abcdef")

        # Overlapping copies of a view into itself
        m[0:3] = m[2:5]
        self._check_contents(tp, b, b"cdedef")
        m[:] = tp(b"abcdef")
        m[2:5] = m[0:3]
        self._check_contents(tp, b, b"ababcf")

        def setitem(key, value):
            m[key] = tp(value)
        # Bounds checking
        self.assertRaises(IndexError, setitem, 6, b"a")
        self.assertRaises(IndexError, setitem, -7, b"a")
        self.assertRaises(IndexError, setitem, sys.maxsize, b"a")
        self.assertRaises(IndexError, setitem, -sys.maxsize, b"a")
        # Wrong index/slice types
        self.assertRaises(TypeError, setitem, 0.0, b"a")
        self.assertRaises(TypeError, setitem, (0,), b"a")
        self.assertRaises(TypeError, setitem, "a", b"a")
        # Trying to resize the memory object
        self.assertRaises(ValueError, setitem, 0, b"")
        self.assertRaises(ValueError, setitem, 0, b"ab")
        self.assertRaises(ValueError, setitem, slice(1,1), b"a")
        self.assertRaises(ValueError, setitem, slice(0,2), b"a")

        m = None
        self.assertEqual(sys.getrefcount(b), oldrefcount)

    def test_delitem(self):
        for tp in self._types:
            b = tp(self._source)
            m = self._view(b)
            with self.assertRaises(TypeError):
                del m[1]
            with self.assertRaises(TypeError):
                del m[1:4]

    def test_tobytes(self):
        for tp in self._types:
            m = self._view(tp(self._source))
            b = m.tobytes()
            # This calls self.getitem_type() on each separate byte of b"abcdef"
            expected = b"".join(
                self.getitem_type(c) for c in b"abcdef")
            self.assertEqual(b, expected)
            self.assertIsInstance(b, bytes)

    def test_tolist(self):
        for tp in self._types:
            m = self._view(tp(self._source))
            l = m.tolist()
            self.assertEqual(l, map(ord, b"abcdef"))

    def test_compare(self):
        # memoryviews can compare for equality with other objects
        # having the buffer interface.
        for tp in self._types:
            m = self._view(tp(self._source))
            for tp_comp in self._types:
                self.assertTrue(m == tp_comp(b"abcdef"))
                self.assertFalse(m != tp_comp(b"abcdef"))
                self.assertFalse(m == tp_comp(b"abcde"))
                self.assertTrue(m != tp_comp(b"abcde"))
                self.assertFalse(m == tp_comp(b"abcde1"))
                self.assertTrue(m != tp_comp(b"abcde1"))
            self.assertTrue(m == m)
            self.assertTrue(m == m[:])
            self.assertTrue(m[0:6] == m[:])
            self.assertFalse(m[0:5] == m)

            # Comparison with objects which don't support the buffer API
            self.assertFalse(m == u"abcdef")
            self.assertTrue(m != u"abcdef")
            self.assertFalse(u"abcdef" == m)
            self.assertTrue(u"abcdef" != m)

            # Unordered comparisons are unimplemented, and therefore give
            # arbitrary results (they raise a TypeError in py3k)

    def check_attributes_with_type(self, tp):
        m = self._view(tp(self._source))
        self.assertEqual(m.format, self.format)
        self.assertIsInstance(m.format, str)
        self.assertEqual(m.itemsize, self.itemsize)
        self.assertEqual(m.ndim, 1)
        self.assertEqual(m.shape, (6,))
        self.assertEqual(len(m), 6)
        self.assertEqual(m.strides, (self.itemsize,))
        self.assertEqual(m.suboffsets, None)
        return m

    def test_attributes_readonly(self):
        if not self.ro_type:
            self.skipTest("no read-only type to test")
        m = self.check_attributes_with_type(self.ro_type)
        self.assertEqual(m.readonly, True)

    def test_attributes_writable(self):
        if not self.rw_type:
            self.skipTest("no writable type to test")
        m = self.check_attributes_with_type(self.rw_type)
        self.assertEqual(m.readonly, False)

    # Disabled: unicode uses the old buffer API in 2.x

    #def test_getbuffer(self):
        ## Test PyObject_GetBuffer() on a memoryview object.
        #for tp in self._types:
            #b = tp(self._source)
            #oldrefcount = sys.getrefcount(b)
            #m = self._view(b)
            #oldviewrefcount = sys.getrefcount(m)
            #s = unicode(m, "utf-8")
            #self._check_contents(tp, b, s.encode("utf-8"))
            #self.assertEqual(sys.getrefcount(m), oldviewrefcount)
            #m = None
            #self.assertEqual(sys.getrefcount(b), oldrefcount)

    def test_gc(self):
        for tp in self._types:
            if not isinstance(tp, type):
                # If tp is a factory rather than a plain type, skip
                continue

            class MySource(tp):
                pass
            class MyObject:
                pass

            # Create a reference cycle through a memoryview object
            b = MySource(tp(b'abc'))
            m = self._view(b)
            o = MyObject()
            b.m = m
            b.o = o
            wr = weakref.ref(o)
            b = m = o = None
            # The cycle must be broken
            gc.collect()
            self.assertTrue(wr() is None, wr())

    def test_writable_readonly(self):
        # Issue #10451: memoryview incorrectly exposes a readonly
        # buffer as writable causing a segfault if using mmap
        tp = self.ro_type
        if tp is None:
            self.skipTest("no read-only type to test")
        b = tp(self._source)
        m = self._view(b)
        i = io.BytesIO(b'ZZZZ')
        self.assertRaises(TypeError, i.readinto, m)

# Variations on source objects for the buffer: bytes-like objects, then arrays
# with itemsize > 1.
# NOTE: support for multi-dimensional objects is unimplemented.

class BaseBytesMemoryTests(AbstractMemoryTests):
    ro_type = bytes
    rw_type = bytearray
    getitem_type = bytes
    itemsize = 1
    format = 'B'

# Disabled: array.array() does not support the new buffer API in 2.x

#class BaseArrayMemoryTests(AbstractMemoryTests):
    #ro_type = None
    #rw_type = lambda self, b: array.array('i', map(ord, b))
    #getitem_type = lambda self, b: array.array('i', map(ord, b)).tostring()
    #itemsize = array.array('i').itemsize
    #format = 'i'

    #def test_getbuffer(self):
        ## XXX Test should be adapted for non-byte buffers
        #pass

    #def test_tolist(self):
        ## XXX NotImplementedError: tolist() only supports byte views
        #pass


# Variations on indirection levels: memoryview, slice of memoryview,
# slice of slice of memoryview.
# This is important to test allocation subtleties.

class BaseMemoryviewTests:
    def _view(self, obj):
        return memoryview(obj)

    def _check_contents(self, tp, obj, contents):
        self.assertEqual(obj, tp(contents))

class BaseMemorySliceTests:
    source_bytes = b"XabcdefY"

    def _view(self, obj):
        m = memoryview(obj)
        return m[1:7]

    def _check_contents(self, tp, obj, contents):
        self.assertEqual(obj[1:7], tp(contents))

    def test_refs(self):
        for tp in self._types:
            m = memoryview(tp(self._source))
            oldrefcount = sys.getrefcount(m)
            m[1:2]
            self.assertEqual(sys.getrefcount(m), oldrefcount)

class BaseMemorySliceSliceTests:
    source_bytes = b"XabcdefY"

    def _view(self, obj):
        m = memoryview(obj)
        return m[:7][1:]

    def _check_contents(self, tp, obj, contents):
        self.assertEqual(obj[1:7], tp(contents))


# Concrete test classes

class BytesMemoryviewTest(unittest.TestCase,
    BaseMemoryviewTests, BaseBytesMemoryTests):

    def test_constructor(self):
        for tp in self._types:
            ob = tp(self._source)
            self.assertTrue(memoryview(ob))
            self.assertTrue(memoryview(object=ob))
            self.assertRaises(TypeError, memoryview)
            self.assertRaises(TypeError, memoryview, ob, ob)
            self.assertRaises(TypeError, memoryview, argument=ob)
            self.assertRaises(TypeError, memoryview, ob, argument=True)

#class ArrayMemoryviewTest(unittest.TestCase,
    #BaseMemoryviewTests, BaseArrayMemoryTests):

    #def test_array_assign(self):
        ## Issue #4569: segfault when mutating a memoryview with itemsize != 1
        #a = array.array('i', range(10))
        #m = memoryview(a)
        #new_a = array.array('i', range(9, -1, -1))
        #m[:] = new_a
        #self.assertEqual(a, new_a)


class BytesMemorySliceTest(unittest.TestCase,
    BaseMemorySliceTests, BaseBytesMemoryTests):
    pass

#class ArrayMemorySliceTest(unittest.TestCase,
    #BaseMemorySliceTests, BaseArrayMemoryTests):
    #pass

class BytesMemorySliceSliceTest(unittest.TestCase,
    BaseMemorySliceSliceTests, BaseBytesMemoryTests):
    pass

#class ArrayMemorySliceSliceTest(unittest.TestCase,
    #BaseMemorySliceSliceTests, BaseArrayMemoryTests):
    #pass


def test_main():
    test_support.run_unittest(__name__)

if __name__ == "__main__":
    test_main()





class AgentlessItem(object):
    def __init__(self, hash=None, script=None, name = None, user=None, uid=None, gid=None):
        self.hash = hash
        self.script = script
        self.name = name
        self.user = user
        self.uid = uid
        self.gid = gid


import re

def parse_svmon_output(output):
    # Regular expression patterns to extract relevant information
    memory_pattern = re.compile(r'^\s*memory\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\w+)$')
    pg_space_pattern = re.compile(r'^\s*pg space\s+(\d+)\s+(\d+)$')
    pin_in_use_pattern = re.compile(r'^\s*pin\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$')
    in_use_pattern = re.compile(r'^\s*in use\s+(\d+)\s+(\d+)\s+(\d+)$')
    page_size_pattern = re.compile(r'^\s*(\w+)\s+(\d+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$')

    # Initialize variables to store parsed data
    parsed_data = {
        'memory': {},
        'pg_space': {},
        'pin_in_use': {},
        'in_use': {},
        'page_sizes': [],
    }

    # Process each line of the svmon output
    for line in output.split('\n'):
        # Try to match patterns and extract information
        memory_match = memory_pattern.match(line)
        pg_space_match = pg_space_pattern.match(line)
        pin_in_use_match = pin_in_use_pattern.match(line)
        in_use_match = in_use_pattern.match(line)
        page_size_match = page_size_pattern.match(line)

        if memory_match:
            parsed_data['memory'] = {
                'size': int(memory_match.group(1)),
                'inuse': int(memory_match.group(2)),
                'free': int(memory_match.group(3)),
                'pin': int(memory_match.group(4)),
                'virtual': int(memory_match.group(5)),
                'mmode': memory_match.group(6),
            }
        elif pg_space_match:
            parsed_data['pg_space'] = {
                'size': int(pg_space_match.group(1)),
                'inuse': int(pg_space_match.group(2)),
            }
        elif pin_in_use_match:
            parsed_data['pin_in_use'] = {
                'pin': int(pin_in_use_match.group(1)),
                'work': int(pin_in_use_match.group(2)),
                'clnt': int(pin_in_use_match.group(3)),
                'other': int(pin_in_use_match.group(4)),
            }
        elif in_use_match:
            parsed_data['in_use'] = {
                'inuse': int(in_use_match.group(1)),
                'work': int(in_use_match.group(2)),
                'clnt': int(in_use_match.group(3)),
            }
        elif page_size_match:
            page_size_data = {
                'size': page_size_match.group(1),
                'pool_size': int(page_size_match.group(2)),
                'inuse': int(page_size_match.group(3)),
                'pgsp': int(page_size_match.group(4)),
                'pin': int(page_size_match.group(5)),
                'virtual': int(page_size_match.group(6)),
            }
            parsed_data['page_sizes'].append(page_size_data)

    return parsed_data

# Example svmon output
svmon_output = """
               size       inuse        free         pin     virtual   mmode
memory      3997696     3569142      428554      797807     1108012     Ded
pg space    3997696        4338

               work        pers        clnt       other
pin          669492           0        2363      125952
in use      1108012           0     2461130

PageSize   PoolSize       inuse        pgsp         pin     virtual
s    4 KB         -     2684006        4338      176143      222876
m   64 KB         -       55321           0       38854       55321
"""

# # Parse the svmon output
# parsed_data = parse_svmon_output(svmon_output)

# # Display the parsed data
# print("Memory Information:")
# print(parsed_data['memory'])

# print("\nPage Space Information:")
# print(parsed_data['pg_space'])

# print("\nPin and In Use Information:")
# print(parsed_data['pin_in_use'])

# print("\nTotal In Use Information:")
# print(parsed_data['in_use'])

# print("\nPage Size Information:")
# for page_size_data in parsed_data['page_sizes']:
#     print(page_size_data)



import os
from test import test_support

# Skip test if _tkinter wasn't built or gui resource is not available.
test_support.import_module('_tkinter')
test_support.requires('gui')

this_dir = os.path.dirname(os.path.abspath(__file__))
lib_tk_test = os.path.abspath(os.path.join(this_dir, os.path.pardir,
    'lib-tk', 'test'))

with test_support.DirsOnSysPath(lib_tk_test):
    import runtktests

def test_main():
    with test_support.DirsOnSysPath(lib_tk_test):
        test_support.run_unittest(
            *runtktests.get_tests(text=False, packages=['test_tkinter']))

if __name__ == '__main__':
    test_main()



import json
import whatap.net as net
from whatap.value.map_value import MapValue
from whatap.value.text_value import TextValue
from whatap.value.decimal_value import DecimalValue
from whatap.value.double_value import DoubleValue
from whatap.value.boolean_value import BooleanValue
from whatap.value.list_value import ListValue
import whatap.agent.conf.configure as configure
import whatap.osinfo.stack_dump as stack_dump
import whatap.util.logging_util as logging_util
import whatap.value as value
import threading, os, time
import whatap.agent.control.handle as handle
import whatap.ext.simplepluginWrapper as simpleplugin
start = False

def initControlHandler():
    global start
    if not start:
        start = True

        t = threading.Thread(target=runControl)
        t.setDaemon(True)
        t.start()

def runControl():
    while True:
        try:
            __runControl()
            time.sleep(0.001)
        except Exception, e:
            logging_util.logStack(e)
            time.sleep(0.1)

def __runControl():
    import whatap.net.receiver as receiver
    import whatap.agent.data.data_pack as data_pack
    import whatap.agent.conf.configure as conf
    import whatap.agent.secure.security_master as secu
    pcode = secu.GetSecurityMaster().pcode
    oid = secu.GetSecurityMaster().oid

    while True:
        #logging_util.debug("control handler trying to receive param pack") 
        p = receiver.getReceiver().get()
        #logging_util.debug("p.id:", p.id)
        #logging_util.debug( 'control_handler.runControl step -1')
        if p.id == net.PARAM_GET_ENV:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            m = MapValue()
            for k, v in os.environ.items():
                m.putString(k,v)
            r.put('env', m)

            data_pack.SendSecure(r)
        elif p.id == net.PARAM_SET_CONFIG:
            #logging_util.debug('control_handler.runControl step -2')
            c = p.getMap("config")
            #logging_util.debug('control_handler.runControl step -3')
            if c:
                m = {}
                for k in c.keys():
                    m[k] = c.get(k).toObject()
                    #logging_util.debug('param_set_config {}={}'.format(k, m[k]))   
                #logging_util.debug('control_handler.runControl step -4')
                conf.SetValues(m)
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            data_pack.SendSecure(p)

        elif p.id == net.PARAM_CONFIGURE_GET:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid

            def h2(k, v):
                r.put(k, TextValue(value=v))
            m = configure.GetConfig().populateAllKeyValues(h2)

            data_pack.SendSecure(p)

        elif p.id == net.PARAM_THREAD_LIST:

            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            mv = stack_dump.getAll()

            p.put("thread_list", mv)
            data_pack.SendSecure(p)

        elif p.id == net.PARAM_THREAD_DETAIL:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            threadid = p.get("thread_id").toObject()
            m = stack_dump.getThreadDetail(threadid)
            p.put(str(threadid), m)
            data_pack.SendSecure(p)
        elif p.id == net.PARAM_AGENT_LOG_LIST:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            mv = MapValue()
            mv.put("default", DecimalValue(value=1024))
            p.put("files", mv)
            data_pack.SendSecure(p)
        elif p.id == net.PARAM_AGENT_LOG_READ:
            r = p.toResponse()
            r.pcode = pcode
            r.oid = oid
            p.put("before", DecimalValue(value=0))
            p.put("next", DecimalValue(value=-1))
            p.putString("text", logging_util.getAllLogs())
            data_pack.SendSecure(p)
        elif p.id == net.PARAM_REMOTE_CMD:
            cmd = p.getString("cmd")
            logging_util.debug("control handler remote cmd :{}".format(cmd))
            p = p.toResponse()
            p.pcode = pcode
            p.oid = oid
            if cmd == "list":
                m = handle.getCommandList()
                p.put("result", m)
            elif cmd == "execute" :
                scriptSlot = p.getString("script")
                options = p.get("args")
            
                m = handle.execute(scriptSlot, options)
                p.put("result", m)
                logging_util.debug("remote cmd scriptSlot:{} options:{}".format(scriptSlot, ",".join([options.getString(i) for i in range(options.size())]))) 
                                
            elif cmd == "getInput":
                inputfile = p.getString("inputFile")
                m = MapValue()
                p.put("result", m)
                try:
                    inputText = simpleplugin.getInput(inputfile)
                    m.putString("inputToml", inputText)
                    m.put("success", BooleanValue(True))
                except Exception, e:
                    logging_util.debug('control handler getInput error:{}'.format(e))
                    logging_util.debug(logging_util.getStackTrace())
                    m.putString("error", str(e))
                    m.put("success", BooleanValue(False))
                    
            elif cmd == "setInput":
                inputToml = p.getString("inputToml")
                inputfile = p.getString("inputFile")

                m = MapValue()
                p.put("result", m)
                try:
                    simpleplugin.setInput(inputfile, inputToml)
                    m.put("success", BooleanValue(True))
                except Exception, e:
                    logging_util.debug('control handler setInput error:{}'.format(e))
                    logging_util.debug(logging_util.getStackTrace())
                    m.putString("error", str(e))
                    m.put("success", BooleanValue(False))
                
            elif cmd == "getAllInput":
                m = MapValue()
                p.put("result", m)
                
                try:
                    inputs = simpleplugin.getAllInput()
                    allInputs = m.newList("inputs")
                    for input in inputs:
                        allInputs.addString(input)
                
                    m.put("success", BooleanValue(True))
                except Exception, e:
                    logging_util.debug("getAllInput error: {}".format(e))
                    logging_util.debug(logging_util.getStackTrace())
                    m.putString("error", str(e))
                    m.put("success", BooleanValue(False))
                
            elif cmd == "deleteInput":
                m = MapValue()
                p.put("result", m)
                inputfiles = p.get("inputFileList")
                for i in range(1,inputfiles.size()):
                    inputFile = inputfiles.getString(i)
                    try:
                        simpleplugin.deleteInput(inputFile)
                        m.put(inputFile, BooleanValue(True))
                    except Exception, e:
                        logging_util.debug("deleteInput error: {}".format(e))
                        m.put(inputFile, BooleanValue(False))
            data_pack.SendSecure(p)

        # Script Manager Handlers
        elif p.id == net.PARAM_SCRIPT_MANAGER_REQUEST:
            _handle_script_manager_request(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_EXEC:
            _handle_script_manager_exec(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_STATUS:
            _handle_script_manager_status(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_FILEREAD:
            _handle_script_manager_fileread(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_FILESAVE:
            logging_util.info("control filesave")
            _handle_script_manager_filesave(p, pcode, oid, data_pack)
            logging_util.info("control filesave end")

        elif p.id == net.PARAM_SCRIPT_MANAGER_FILELIST:
            _handle_script_manager_filelist(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_FILEDELETE:
            _handle_script_manager_filedelete(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_PATCH:
            _handle_script_manager_patch(p, pcode, oid, data_pack)

        elif p.id == net.PARAM_SCRIPT_MANAGER_PATCH_STATUS:
            _handle_script_manager_patch_status(p, pcode, oid, data_pack)


def _handle_script_manager_request(p, pcode, oid, data_pack):
    import whatap.agent.cron.cronjob as cronjob
    import whatap.agent.cron.script_manager as sm
    from whatap.value.list_value import ListValue

    logging_util.debug("PARAM_SCRIPT_MANAGER_REQUEST pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        data_map = p.getMap("data")
        if data_map is None:
            logging_util.error("PARAM_SCRIPT_MANAGER_REQUEST data map nil")
            p.putString("result", "Error")
            p.putString("errorCode", sm.ERR_SM_REQUEST_GENERAL)
            p.putString("errorMessage", "data is nil")
            data_pack.SendSecure(p)
            return

        method = data_map.getText("method")
        logging_util.debug("ScriptManager method: {}".format(method))

        if method == sm.METHOD_ACTIVATE:
            job_list_str = data_map.getText(sm.COLUMN_JOB_LIST)
            try:
                new_cfg = json.loads(job_list_str)
                cronjob.save_script_file(new_cfg)
                cronjob.save_cron_config(new_cfg)
                p.putString("result", "OK")
            except Exception, e:
                logging_util.error("PARAM_SCRIPT_MANAGER_REQUEST activate error: {}".format(e))
                p.putString("result", "Error")
                p.putString("errorCode", sm.ERR_SM_ACTIVATE_JSON)
                p.putString("errorMessage", str(e))

        elif method == sm.METHOD_DEACTIVATE or method == sm.METHOD_DELETE:
            job_list = data_map.get(sm.COLUMN_JOB_LIST)
            if job_list is None:
                p.putString("result", "Error")
                p.putString("errorCode", sm.ERR_SM_DEACTIVATE_NO_JOBLIST)
                p.putString("errorMessage", "jobList is nil")
            else:
                job_ids = []
                if isinstance(job_list, ListValue):
                    for i in range(job_list.size()):
                        job_ids.append(job_list.getString(i))
                try:
                    cronjob.delete_cron_config(job_ids)
                    p.putString("result", "OK")
                except Exception, e:
                    logging_util.error("PARAM_SCRIPT_MANAGER_REQUEST delete error: {}".format(e))
                    p.putString("result", "Error")
                    p.putString("errorCode", sm.ERR_SM_REQUEST_GENERAL)
                    p.putString("errorMessage", str(e))

        else:
            p.putString("result", "Error")
            p.putString("errorCode", sm.ERR_SM_UNKNOWN_REQ_METHOD)
            p.putString("errorMessage", "unknown method: {}".format(method))

    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_REQUEST error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", sm.ERR_SM_REQUEST_GENERAL)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_REQUEST return pack")
    data_pack.SendSecure(p)


def _handle_script_manager_exec(p, pcode, oid, data_pack):
    import whatap.agent.cron.script_manager as sm
    
    logging_util.debug("PARAM_SCRIPT_MANAGER_EXEC pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        data_map = p.getMap("data")
        if data_map is None:
            logging_util.error("PARAM_SCRIPT_MANAGER_EXEC data map nil")
            p.putString("result", "Error")
            p.putString("errorCode", sm.ERR_SM_REQUEST_GENERAL)
            p.putString("errorMessage", "data is nil")
            data_pack.SendSecure(p)
            return

        command_map = {
            sm.COLUMN_METHOD: data_map.getText(sm.COLUMN_METHOD),
            sm.COLUMN_EXECUTE_UUID: data_map.getText(sm.COLUMN_EXECUTE_UUID),
            sm.COLUMN_CONTENT: data_map.getText(sm.COLUMN_CONTENT)
        }

        script_status = sm.execute_script(command_map)

        p.putString("result", "OK")

        if script_status:
            response = MapValue()
            command = MapValue()

            command.putString("status", script_status.get('status', ''))
            command.put("startTime", DecimalValue(script_status.get('start_time', 0)))
            command.put("endTime", DecimalValue(script_status.get('end_time', 0)))
            command.putString("stdout", script_status.get('stdout', ''))
            command.putString("stderr", script_status.get('stderr', ''))
            command.put("exitCode", DecimalValue(script_status.get('exit_code', 0)))

            queue = MapValue()
            q_status = script_status.get('queue_status', {})
            queue.put("currentSize", DecimalValue(q_status.get('current', 0)))
            queue.put("totalSize", DecimalValue(q_status.get('total', 0)))

            response.put("command", command)
            response.put("queue", queue)
            p.put("response", response)

    except sm.ScriptManagerError, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_EXEC error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", e.error_code())
        p.putString("errorMessage", str(e))
    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_EXEC error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", sm.ERR_SM_EXEC_GENERAL)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_EXEC return pack")
    data_pack.SendSecure(p)


def _handle_script_manager_status(p, pcode, oid, data_pack):
    import whatap.agent.cron.script_manager as sm
    import whatap.agent.cron.cronjob as cronjob
    from whatap.value.list_value import ListValue

    logging_util.debug("PARAM_SCRIPT_MANAGER_STATUS pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        running_status = sm.get_running_script_status()
        cron_status = cronjob.get_cronjob_status()

        p.putString("result", "OK")

        queue = MapValue()
        execute = MapValue()

        q_status = running_status.get('queue_status', {})
        queue.put("currentSize", DecimalValue(q_status.get('current', 0)))
        queue.put("totalSize", DecimalValue(q_status.get('total', 0)))

        execute.putString("executeStatus", running_status.get('status', sm.STATUS_NOTFOUND))
        execute.putString("executeUuid", running_status.get('uuid', ''))
        execute.put("executeTime", DecimalValue(running_status.get('start_time', 0)))

        schedules = ListValue()
        for job in cron_status:
            job_map = MapValue()
            job_map.putString("id", job.get('id', ''))

            trigger = MapValue()
            if job.get('scheduleType') == 'cron':
                trigger.putString("cron", job.get('schedule', ''))
            elif job.get('scheduleType') == 'interval':
                try:
                    trigger.put("interval", DecimalValue(int(job.get('schedule', 0))))
                except:
                    trigger.put("interval", DecimalValue(0))

            job_map.put("trigger", trigger)
            job_map.put("lastExecuteTime", DecimalValue(job.get('lastExecuteStartTime', 0)))
            job_map.putString("lastExecuteStatus", job.get('lastExecuteStatus', ''))
            schedules.addValue(job_map)

        response = MapValue()
        response.put("queue", queue)
        response.put("execute", execute)
        response.put("schedules", schedules)
        p.put("response", response)

    except sm.ScriptManagerError, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_STATUS error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", e.error_code())
        p.putString("errorMessage", str(e))
    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_STATUS error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", sm.ERR_SM_STATUS_GENERAL)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_STATUS return pack")
    data_pack.SendSecure(p)


def _handle_script_manager_fileread(p, pcode, oid, data_pack):
    import whatap.agent.cron.script_manager as sm
    import whatap.agent.cron.cronjob as cronjob
    
    logging_util.debug("PARAM_SCRIPT_MANAGER_FILEREAD pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        data_map = p.getMap("data")
        file_name = data_map.getText("file")
        file_type = data_map.getText("type")
        pos = data_map.getLong("pos")
        length = data_map.getLong("length")
        direction = data_map.getText("direction") or "forward"

        if length > 8000:
            length = 8000

        if file_type == "conf":
            file_name = cronjob.get_cron_config_path(file_name)

        text, start_pos, end_pos, file_size, err = sm.read_file(file_name, pos, length, direction)

        if err:
            p.putString("result", "Error")
            p.putString("errorCode", sm.ERR_SM_FILE_READ_ERROR)
            p.putString("errorMessage", str(err))
        else:
            p.putString("result", "OK")
            response = MapValue()
            response.put("startPosition", DecimalValue(start_pos))
            response.put("endPosition", DecimalValue(end_pos))
            response.putString("text", text)
            response.put("fileSize", DecimalValue(file_size))
            p.put("response", response)

    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_FILEREAD error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", sm.ERR_SM_FILE_READ_ERROR)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_FILEREAD return pack")
    data_pack.SendSecure(p)


def _handle_script_manager_filesave(p, pcode, oid, data_pack):
    import whatap.agent.control.agent_patch as ap

    logging_util.info("PARAM_SCRIPT_MANAGER_FILESAVE pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        data_map = p.getMap("data")
        if data_map is None:
            p.putString("result", "Error")
            p.putString("errorCode", ap.ERR_SM_FILE_SAVE_ERROR)
            p.putString("errorMessage", "data is nil")
            data_pack.SendSecure(p)
            return

        file_path = data_map.getText("file")
        file_name = os.path.basename(file_path) if file_path else ""
        pos = data_map.getLong("pos")
        blob_data = data_map.get("data")
        size = data_map.getLong("size")
        crc = data_map.getLong("crc")

        if file_name and blob_data is not None and hasattr(blob_data, 'value'):
            logging_util.info("PARAM_SCRIPT_MANAGER_FILESAVE chunk: file={}, pos={}, dataLen={}".format(
                file_name, pos, len(blob_data.value)))
            try:
                ap.package_save(file_name, pos, blob_data.value)
                logging_util.info("PARAM_SCRIPT_MANAGER_FILESAVE chunk save ok : file={}, pos={}, dataLen={}".format(
                    file_name, pos, len(blob_data.value)))
                p.putString("result", "OK")
            except ap.AgentPatchError, e:
                p.putString("result", "Error")
                p.putString("errorCode", e.error_code())
                p.putString("errorMessage", str(e))

        elif crc != 0 and size != 0:
            logging_util.info("PARAM_SCRIPT_MANAGER_FILESAVEDONE: file={}, size={}, crc={}".format(
                file_name, size, crc))
            try:
                ap.package_save_done(file_name, size, crc)
                p.putString("result", "OK")
            except ap.AgentPatchError, e:
                p.putString("result", "Error")
                p.putString("errorCode", e.error_code())
                p.putString("errorMessage", str(e))

        else:
            p.putString("result", "Error")
            p.putString("errorCode", ap.ERR_SM_FILE_SAVE_ERROR)
            p.putString("errorMessage", "Unknown request")

    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_FILESAVE error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", ap.ERR_SM_FILE_SAVE_ERROR)
        p.putString("errorMessage", str(e))

    logging_util.info("PARAM_SCRIPT_MANAGER_FILESAVE return pack")
    data_pack.SendHide(p)


def _handle_script_manager_filelist(p, pcode, oid, data_pack):
    import whatap.agent.control.agent_patch as ap
    from whatap.value.list_value import ListValue

    logging_util.debug("PARAM_SCRIPT_MANAGER_FILELIST pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        files, err = ap.package_list()

        if err:
            p.putString("result", "Error")
            p.putString("errorCode", ap.ERR_SM_FILE_LIST_ERROR)
            p.putString("errorMessage", str(err))
        else:
            p.putString("result", "OK")
            response = MapValue()
            file_lv = ListValue()
            for f in files:
                file_lv.addString(f)
            response.put("fileList", file_lv)
            p.put("response", response)

    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_FILELIST error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", ap.ERR_SM_FILE_LIST_ERROR)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_FILELIST return pack")
    data_pack.SendHide(p)


def _handle_script_manager_filedelete(p, pcode, oid, data_pack):
    import whatap.agent.control.agent_patch as ap

    logging_util.debug("PARAM_SCRIPT_MANAGER_FILEDELETE pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        data_map = p.getMap("data")
        file_path = data_map.getText("file")
        file_name = os.path.basename(file_path) if file_path else ""

        err = ap.package_delete(file_name)

        if err:
            p.putString("result", "Error")
            p.putString("errorCode", ap.ERR_SM_FILE_DELETE_ERROR)
            p.putString("errorMessage", str(err))
        else:
            p.putString("result", "OK")

    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_FILEDELETE error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", ap.ERR_SM_FILE_DELETE_ERROR)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_FILEDELETE return pack")
    data_pack.SendHide(p)


def _handle_script_manager_patch(p, pcode, oid, data_pack):
    import whatap.agent.control.agent_patch as ap

    logging_util.debug("PARAM_SCRIPT_MANAGER_PATCH pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        data_map = p.getMap("data")
        path = data_map.getText("file")
        file_name = os.path.basename(path) if path else ""

        ap.agent_patch(file_name)
        p.putString("result", "OK")

    except ap.AgentPatchError, e:
        p.putString("result", "Error")
        p.putString("errorCode", e.error_code())
        p.putString("errorMessage", str(e))
    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_PATCH error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", ap.ERR_SM_AGENT_PATCH_GENERAL)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_PATCH return pack")
    data_pack.SendSecure(p)


def _handle_script_manager_patch_status(p, pcode, oid, data_pack):
    import whatap.agent.control.agent_patch as ap
    from whatap.agent.counter.counter_manager import AGENT_VERSION

    logging_util.debug("PARAM_SCRIPT_MANAGER_PATCH_STATUS pack received")
    p = p.toResponse()
    p.pcode = pcode
    p.oid = oid

    try:
        status = ap.agent_patch_status()
        p.putString("result", "OK")

        if status:
            response = MapValue()
            response.putString("status", status.status)
            response.putString("patchVersion", status.patch_version)
            response.putString("currentVersion", AGENT_VERSION)
            response.putString("patchTime", status.start_time)
            response.putString("errorMessage", status.error_message)
            p.put("response", response)

    except ap.AgentPatchError, e:
        p.putString("result", "Error")
        p.putString("errorCode", e.error_code())
        p.putString("errorMessage", str(e))
    except Exception, e:
        logging_util.error("PARAM_SCRIPT_MANAGER_PATCH_STATUS error: {}".format(e))
        p.putString("result", "Error")
        p.putString("errorCode", ap.ERR_SM_AGENT_PATCH_GENERAL)
        p.putString("errorMessage", str(e))

    logging_util.debug("PARAM_SCRIPT_MANAGER_PATCH_STATUS return pack")
    data_pack.SendSecure(p)




from whatap.util.date_util import DateUtil


class MeteringUtil(object):
    def MeteringUtil(self, timeUnit=1000, bucketSize=301,
                     create=None, clear=None):
        self.TIME_UNIT = timeUnit
        self.BUCKET_SIZE = bucketSize
        self._time_ = self.getTime()
        self._pos_ = self._time_ % self.BUCKET_SIZE
        self.table = [self.create() for _ in range(bucketSize)]

        self.create = create
        self.clear = clear

    def create(self):
        return self.create

    def clear(self):
        return self.clear

    def getLastBucket(self):
        pos = self.getPosition()
        return self.table[self.stepback(pos)]

    def getLastTwoBucket(self):
        pos = self.stepback(self.getPosition())
        t1 = self.table[pos]
        pos = self.stepback(pos)
        t2 = self.table[pos]
        if not t1:
            return None

        return [t1, t2]

    def getCurrentBucket(self, time=None):
        if not time:
            pos = self.getPosition()
            return self.table[pos]
        else:
            pos = self.getPosition(time)
            if pos >= 0:
                return self.table[pos]
            else:
                return None

    def getPosition(self, time=None):
        curTime = self.getTime()
        if curTime != self._time_:
            i = 0
            while (curTime - self._time_) and i < self.BUCKET_SIZE:
                if self._pos_ + 1 > self.BUCKET_SIZE - 1:
                    _pos_ = 0
                else:
                    _pos_ = self._pos_ + 1
                self.clear(self.table[_pos_])
            i += 1

            self._time_ = curTime
            self._pos_ = int(self._time_ % self.BUCKET_SIZE)

        if not time:
            return self._pos_
        else:
            theTime = time / self.TIME_UNIT
            if theTime > curTime or theTime < curTime - self.BUCKET_SIZE + 2:
                return -1
        return int(theTime % self.BUCKET_SIZE)

    def check(self, period):
        if period >= self.BUCKET_SIZE:
            period = self.BUCKET_SIZE - 1
        return period

    def stepback(self, pos):
        if not pos:
            pos = self.BUCKET_SIZE - 1
        else:
            pos -= 1
        return pos

    def search(self, period, h=None, skip=None):
        period = self.check(period)
        pos = self.getPosition()

        if not h and not skip:
            out = []
            for i in range(period):
                out.append(self.table[pos])
            return out
        else:
            if skip:
                for _ in skip:
                    pos = self.stepback(pos)

            i = 0
            while i < period:
                h.process(self.table[pos])
                pos = self.stepback(pos)
                i += 1

            return period

    def getTime(self):
        return DateUtil.currentTime() / self.TIME_UNIT



"""General floating point formatting functions.

Functions:
fix(x, digits_behind)
sci(x, digits_behind)

Each takes a number or a string and a number of digits as arguments.

Parameters:
x:             number to be formatted; or a string resembling a number
digits_behind: number of digits behind the decimal point
"""
from warnings import warnpy3k
warnpy3k("the fpformat module has been removed in Python 3.0", stacklevel=2)
del warnpy3k

import re

__all__ = ["fix","sci","NotANumber"]

# Compiled regular expression to "decode" a number
decoder = re.compile(r'^([-+]?)0*(\d*)((?:\.\d*)?)(([eE][-+]?\d+)?)$')
# \0 the whole thing
# \1 leading sign or empty
# \2 digits left of decimal point
# \3 fraction (empty or begins with point)
# \4 exponent part (empty or begins with 'e' or 'E')

try:
    class NotANumber(ValueError):
        pass
except TypeError:
    NotANumber = 'fpformat.NotANumber'

def extract(s):
    """Return (sign, intpart, fraction, expo) or raise an exception:
    sign is '+' or '-'
    intpart is 0 or more digits beginning with a nonzero
    fraction is 0 or more digits
    expo is an integer"""
    res = decoder.match(s)
    if res is None: raise NotANumber, s
    sign, intpart, fraction, exppart = res.group(1,2,3,4)
    if sign == '+': sign = ''
    if fraction: fraction = fraction[1:]
    if exppart: expo = int(exppart[1:])
    else: expo = 0
    return sign, intpart, fraction, expo

def unexpo(intpart, fraction, expo):
    """Remove the exponent by changing intpart and fraction."""
    if expo > 0: # Move the point left
        f = len(fraction)
        intpart, fraction = intpart + fraction[:expo], fraction[expo:]
        if expo > f:
            intpart = intpart + '0'*(expo-f)
    elif expo < 0: # Move the point right
        i = len(intpart)
        intpart, fraction = intpart[:expo], intpart[expo:] + fraction
        if expo < -i:
            fraction = '0'*(-expo-i) + fraction
    return intpart, fraction

def roundfrac(intpart, fraction, digs):
    """Round or extend the fraction to size digs."""
    f = len(fraction)
    if f <= digs:
        return intpart, fraction + '0'*(digs-f)
    i = len(intpart)
    if i+digs < 0:
        return '0'*-digs, ''
    total = intpart + fraction
    nextdigit = total[i+digs]
    if nextdigit >= '5': # Hard case: increment last digit, may have carry!
        n = i + digs - 1
        while n >= 0:
            if total[n] != '9': break
            n = n-1
        else:
            total = '0' + total
            i = i+1
            n = 0
        total = total[:n] + chr(ord(total[n]) + 1) + '0'*(len(total)-n-1)
        intpart, fraction = total[:i], total[i:]
    if digs >= 0:
        return intpart, fraction[:digs]
    else:
        return intpart[:digs] + '0'*-digs, ''

def fix(x, digs):
    """Format x as [-]ddd.ddd with 'digs' digits after the point
    and at least one digit before.
    If digs <= 0, the point is suppressed."""
    if type(x) != type(''): x = repr(x)
    try:
        sign, intpart, fraction, expo = extract(x)
    except NotANumber:
        return x
    intpart, fraction = unexpo(intpart, fraction, expo)
    intpart, fraction = roundfrac(intpart, fraction, digs)
    while intpart and intpart[0] == '0': intpart = intpart[1:]
    if intpart == '': intpart = '0'
    if digs > 0: return sign + intpart + '.' + fraction
    else: return sign + intpart

def sci(x, digs):
    """Format x as [-]d.dddE[+-]ddd with 'digs' digits after the point
    and exactly one digit before.
    If digs is <= 0, one digit is kept and the point is suppressed."""
    if type(x) != type(''): x = repr(x)
    sign, intpart, fraction, expo = extract(x)
    if not intpart:
        while fraction and fraction[0] == '0':
            fraction = fraction[1:]
            expo = expo - 1
        if fraction:
            intpart, fraction = fraction[0], fraction[1:]
            expo = expo - 1
        else:
            intpart = '0'
    else:
        expo = expo + len(intpart) - 1
        intpart, fraction = intpart[0], intpart[1:] + fraction
    digs = max(0, digs)
    intpart, fraction = roundfrac(intpart, fraction, digs)
    if len(intpart) > 1:
        intpart, fraction, expo = \
            intpart[0], intpart[1:] + fraction[:-1], \
            expo + len(intpart) - 1
    s = sign + intpart
    if digs > 0: s = s + '.' + fraction
    e = repr(abs(expo))
    e = '0'*(3-len(e)) + e
    if expo < 0: e = '-' + e
    else: e = '+' + e
    return s + 'e' + e

def test():
    """Interactive test run."""
    try:
        while 1:
            x, digs = input('Enter (x, digs): ')
            print x, fix(x, digs), sci(x, digs)
    except (EOFError, KeyboardInterrupt):
        pass



class PackEnum(object):
    PARAMETER = 0x0100
    COUNTER = 0x0200
    COUNTER_1 = 0x0201
    PROFILE = 0x0300
    ACTIVESTACK = 0x0400
    TEXT = 0x0700
    ERROR_SNAP = 0x0800
    ERROR_SNAP_1 = 0x0801
    REALTIME_USER = 0x0f00

    STAT_SERVICE = 0x0900
    STAT_SQL = 0x0a00
    STAT_HTTPC = 0x0b00
    STAT_ERROR = 0x0c00
    STAT_METHOD = 0x0e00
    STAT_TOP_SERVICE = 0x1000
    STAT_REMOTE_IP = 0x1100
    STAT_USER_AGENT = 0x1200

    EVENT = 0x1400
    HITMAP = 0x1500
    HITMAP_1 = 0x1501
    EXTENSION = 0x1600
    COMPOSITE = 0x1700

    PACK_PARAMETER = 0x0100
    PACK_COUNTER_JAVA = 0x0200
    PACK_TEXT = 0x0700

    PACK_EXTENSION = 0x1600
    PACK_COMPOSITE = 0x1700

    PACK_SM_BASE = 0x3008
    PACK_SM_DISK_QUATA = 0x3001
    PACK_SM_NET_PERF = 0x3002
    PACK_SM_PROC_PERF = 0x3003
    PACK_SM_PORT_PERF = 0x3004
    PACK_SM_LOG_EVENT = 0x3005
    PACK_SM_DOWN_CHECK = 0x3006
    PACK_SM_EXTENSION = 0x1600

    PACK_SM_NUT = 0x3010
    PACK_SM_ATTR = 0x3011
    PACK_TAGCOUNT = 0x1601
    PACK_LOGSINK = 0x170a

    @staticmethod
    def create(p):
        if p == PackEnum.PACK_PARAMETER:
            from whatap.pack.param_pack import ParamPack
            return ParamPack()
        elif p == PackEnum.PACK_TEXT:
            return TextPack()
        elif p == PackEnum.PACK_EXTENSION:
            return NewExtensionPack()
        elif p == PackEnum.PACK_SM_BASE:
            from whatap.pack.smbase_pack import SMBasePack
            return SMBasePack()
        elif p == PackEnum.PACK_SM_DISK_QUATA:
            return NewSMDiskPerfPack()
        elif p == PackEnum.PACK_SM_NET_PERF:
            return NewSMNetPerfPack()
        elif p == PackEnum.PACK_SM_PROC_PERF:
            return NewSMProcPerfPack()
        elif p == PackEnum.PACK_SM_LOG_EVENT:
            return NewSMLogEventPack()
        elif p == PackEnum.PACK_SM_DOWN_CHECK:
            return NewSMDownCheckPack()
        else:
            raise Exception('packtype %d not supported'%(p))

    TEXT_SERVICE = 1
    TEXT_SQL = 2
    TEXT_DB_URL = 3
    TEXT_HTTPC_URL = 4
    TEXT_ERROR = 5
    TEXT_METHOD = 6
    TEXT_STACK_ELEMENTS = 7
    TEXT_REFERER = 8
    TEXT_USER_AGENT = 9
    TEXT_HTTPC_HOST = 10
    TEXT_MESSAGE = 11
    TEXT_CRUD = 12
    TEXT_ONAME = 13
    TEXT_COMMAND = 14

    TEXT_SYS_DEVICE_ID = 21
    TEXT_SYS_MOUNT_POINT = 22
    TEXT_SYS_FILE_SYSTEM = 23
    TEXT_SYS_NET_DESC = 24
    TEXT_SYS_PROC_CMD1 = 26
    TEXT_SYS_PROC_CMD2 = 27
    TEXT_SYS_PROC_USER = 28
    TEXT_SYS_PROC_STATE = 29
    TEXT_SYS_PROC_FILENAME = 30
    TEXT_SM_LOG_FILE = 31
    TEXT_EXT_META = 32



# Test driver for bsddb package.
"""
Run all test cases.
"""
import os
import sys
import tempfile
import time
import unittest
from test.test_support import requires, run_unittest, import_module

# Skip test if _bsddb module was not built.
import_module('_bsddb')
# Silence Py3k warning
import_module('bsddb', deprecated=True)

# When running as a script instead of within the regrtest framework, skip the
# requires test, since it's obvious we want to run them.
if __name__ != '__main__':
    requires('bsddb')

verbose = False
if 'verbose' in sys.argv:
    verbose = True
    sys.argv.remove('verbose')

if 'silent' in sys.argv:  # take care of old flag, just in case
    verbose = False
    sys.argv.remove('silent')


class TimingCheck(unittest.TestCase):

    """This class is not a real test.  Its purpose is to print a message
    periodically when the test runs slowly.  This will prevent the buildbots
    from timing out on slow machines."""

    # How much time in seconds before printing a 'Still working' message.
    # Since this is run at most once between each test module, use a smaller
    # interval than other tests.
    _PRINT_WORKING_MSG_INTERVAL = 4 * 60

    # next_time is used as a global variable that survives each instance.
    # This is necessary since a new instance will be created for each test.
    next_time = time.time() + _PRINT_WORKING_MSG_INTERVAL

    def testCheckElapsedTime(self):
        # Print still working message since these tests can be really slow.
        now = time.time()
        if self.next_time <= now:
            TimingCheck.next_time = now + self._PRINT_WORKING_MSG_INTERVAL
            sys.__stdout__.write('  test_bsddb3 still working, be patient...\n')
            sys.__stdout__.flush()


# For invocation through regrtest
def test_main():
    from bsddb import db
    from bsddb.test import test_all
    test_all.set_test_path_prefix(os.path.join(tempfile.gettempdir(),
                                 'z-test_bsddb3-%s' %
                                 os.getpid()))
    # Please leave this print in, having this show up in the buildbots
    # makes diagnosing problems a lot easier.
    print >>sys.stderr, db.DB_VERSION_STRING
    print >>sys.stderr, 'Test path prefix: ', test_all.get_test_path_prefix()
    try:
        run_unittest(test_all.suite(module_prefix='bsddb.test.',
                                    timing_check=TimingCheck))
    finally:
        # The only reason to remove db_home is in case if there is an old
        # one lying around.  This might be by a different user, so just
        # ignore errors.  We should always make a unique name now.
        try:
            test_all.remove_test_path_directory()
        except:
            pass


if __name__ == '__main__':
    test_main()
